'---------------------------------------------------------------------
' 
'  Copyright (C) xxxxxxxx.  All rights reserved.
' 
'THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
'KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
'IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
'PARTICULAR PURPOSE.
'---------------------------------------------------------------------


Option Explicit

' Constants for enroll context
Const ContextUser                       = 1
Const ContextMachine                    = 2
Const ContextAdministratorForceMachine  = 3

' Constants for enrollment status
Const Enrolled              = 1
Const EnrollPended          = 2

' Command line parameters and defaults
' --------------------------------------------------------------
' /tn
Dim TemplateName		:	TemplateName = "Client Authentication"
' /fn
Dim CertFriendlyName	:	CertFriendlyName = "Test Certificate"
' /desc
Dim CertDescription		:	CertDescription = "Enrolled from VBScript CertEnroll Sample"
' /context
Dim RequestContext		:	RequestContext = 1 

Dim X509Enrollment
Dim X509EnrollmentStatus
Dim EnrollmentStatus

Dim args
Dim i

' Check to see if they need help
Set args = WScript.Arguments
if Err.Number <> 0 Then
    Wscript.Echo("Accessing WScript.Arguments: " & Err.Description)
    Err.Clear
End if

if args.count = 1 then
    if args(0) = "/?" Then
      Call PrintUsage
      WScript.Quit(0)
    End If
    If strcomp(LEFT(args(0), 2), "/h", NOT_CASE_SENSITIVE) = 0 then
      ' Matched /h* or /H* (/h, /help, /H, /HELP)
      call PrintUsage
      WScript.Quit(0)
    End If
End If


For i = 0 To args.Count - 1
    if strcomp(args(i), "/tn") = 0 then
         TemplateName = args(i+1) 

    elseif strcomp(args(i), "/fn") = 0 then
         CertFriendlyName = args(i+1) 

    elseif strcomp(args(i), "/desc") = 0 then
         CertDescription = args(i+1)

    elseif strcomp(args(i), "/context") = 0 then
         If strcomp(Trim(args(i+1)), "user") = 0 Then 
			RequestContext = ContextUser
         ElseIf strcomp(Trim(args(i+1)), "machine") = 0 Then 
			RequestContext = ContextMachine
         ElseIf strcomp(Trim(args(i+1)), "admin") = 0 Then 
			RequestContext = ContextAdministratorForceMachine
		 Else
		    WScript.Echo("Invalid value for context parameter: " & args(i+1)) 
		    Wscript.Quit(0)
		 End If
    End If
Next

' Create IX509Enrollment
Set X509Enrollment = CreateObject("X509Enrollment.CX509Enrollment")
If (Err.Number <> 0) Then
	WScript.Echo "CertEnroll is only available on Windows Vista and later. You must use XEnroll for Windows XP and prior versions,"
    Wscript.Echo "Cannot Create CertEnroll object: " & Err.Number & Err.Description
    Wscript.Quit(-1)
End If

' Initialize IX509Enrollment
Call X509Enrollment.InitializeFromTemplateName(RequestContext, TemplateName)
If (Err.Number <> 0) Then
    Wscript.Echo Err.Number & Err.Description
    Wscript.Quit(-1)
End If

' Add certificate friendly name 
X509Enrollment.CertificateFriendlyName = CertFriendlyName
If (Err.Number <> 0) Then
    Wscript.Echo Err.Number & Err.Description
    Wscript.Quit(-1)
End If

' Add certificate description
X509Enrollment.CertificateDescription = CertDescription
If (Err.Number <> 0) Then
    Wscript.Echo Err.Number & Err.Description
    Wscript.Quit(-1)
End If

' Enroll
Call X509Enrollment.Enroll()
If (Err.Number <> 0) Then
    Wscript.Echo Err.Number & Err.Description
    Wscript.Quit(-1)
End If

' Get IX509EnrollmentStatus
Set X509EnrollmentStatus = X509Enrollment.Status
If (Err.Number <> 0) Then
    Wscript.Echo Err.Number & Err.Description
    Wscript.Quit(-1)
End If

' Get enrollment status
EnrollmentStatus = X509EnrollmentStatus.Status
If (Err.Number <> 0) Then
    Wscript.Echo Err.Number & Err.Description
    Wscript.Quit(-1)
End If

' Display result
If (EnrollmentStatus = Enrolled) Then
    Wscript.Echo "Cert Issued: " & X509EnrollmentStatus.ErrorText & "--" & X509EnrollmentStatus.Text
Elseif (EnrollmentStatus = EnrollPended) Then
    Wscript.Echo "Request is pending: " & X509EnrollmentStatus.ErrorText & "--" & X509EnrollmentStatus.Text
Else
    Wscript.Echo "Request Failed: " & X509EnrollmentStatus.ErrorText & "--" & X509EnrollmentStatus.Text
End If

Wscript.Quit(0)

' PrintUsage function
Sub PrintUsage
    Wscript.Echo "Usage: cscript.exe CertEnrollSample.vbs /tn ""Template Name"" /fn ""Certificate Friendly Name"" /desc ""Certificate Description"" /context ""Certifcate Request Context (user, machine, admin)"""
    Wscript.Echo "Example: CertEnrollSample.vbs /tn ""Client Authentication"" /fn ""My ClientAuth Cert"" /desc ""Client Authentication Certificate"" /context user"
    Wscript.Quit(-1)
End Sub ' PrintUsage


