' Enrollment CAPICOM script

' Requires the installation of CAPICOM.DLL on the client computer
' Please change the values in strMachineConfig and strUserConfig on lines 29 and 30
'
' This script is provided "As is", no technical assistance will be offered
' 
' Microsoft(C) Corporation 2004
'

Option Explicit
Dim MY_DEBUG
MY_DEBUG = True
If Not MY_DEBUG Then : On Error Resume Next : End If


' ---------- Start Configurable Options ----------------------------
' This application name should include a version number but should remain short.
'   It is used when logging errors and such
Const AppName = "enroll.vbs v01.0.0"

' No default certificate type. Require one from the user.
Const DEFAULT_CERTIFICATE_TYPE = ""

' Modify the value of StrMachineConfig and StrUserConfig:
' You can obtain the value of CAServerName by running certutil -getreg ca\CAServerName
' You can obtain the value of CommonName by running certutil -getreg ca\CommonName

Const strMachineConfig = "CAServerName\CommonName"
Const strUserConfig    = "CAServerName\CommonName"

' Encoding type for the SubjAltName CustomID
' Only one of these should be uncommented
'Const strEncodeType = "{asn}"
Const strEncodeType = "{octet}"
'Const strEncodeType = "{utf8}"

' Put a Unicode (2 byte) or single byte string in the contents of the SubjAltName CustomID
Const bUseUnicode = False

' ---------- End Configurable Options ----------------------------

' ---- START CAPICOM Constants
' Store Location Names (CAPICOM_STORE_LOCATION enumeration)
Const CAPICOM_MEMORY_STORE                                     = 0
Const CAPICOM_LOCAL_MACHINE_STORE                              = 1
Const CAPICOM_CURRENT_USER_STORE                               = 2
Const CAPICOM_ACTIVE_DIRECTORY_USER_STORE                      = 3
Const CAPICOM_SMART_CARD_USER_STORE                            = 4
' Certificate store names
Const CAPICOM_MY_STORE    = "My"
Const CAPICOM_CA_STORE    = "CA"
Const CAPICOM_ROOT_STORE  = "Root"
Const CAPICOM_OTHER_STORE = "AddressBook" 
Const CAPICOM_STORE_OPEN_READ_ONLY = 0
' The CAPICOM_EKU enumeration
Const CAPICOM_EKU_OTHER            = 0
Const CAPICOM_EKU_SERVER_AUTH      = 1
Const CAPICOM_EKU_CLIENT_AUTH      = 2
Const CAPICOM_EKU_CODE_SIGNING     = 3
Const CAPICOM_EKU_EMAIL_PROTECTION = 4
Const CAPICOM_EKU_SMARTCARD_LOGON  = 5
Const CAPICOM_EKU_ENCRYPTING_FILE_SYSTEM = 6
' Cert info
Const CAPICOM_CERT_INFO_SUBJECT_EMAIL_NAME    = 2
Const CAPICOM_CERT_INFO_SUBJECT_UPN           = 4
Const CAPICOM_CERT_INFO_SUBJECT_DNS_NAME      = 6
Const CAPICOM_ENCODED_DATA_FORMAT_MULTI_LINES = 1
Const CAPICOM_CERT_POLICIES_OID               = "2.5.29.32"
' Find
Const CAPICOM_CERTIFICATE_FIND_SHA1_HASH                       = 0
Const CAPICOM_CERTIFICATE_FIND_SUBJECT_NAME                    = 1
Const CAPICOM_CERTIFICATE_FIND_ISSUER_NAME                     = 2
Const CAPICOM_CERTIFICATE_FIND_ROOT_NAME                       = 3
Const CAPICOM_CERTIFICATE_FIND_TEMPLATE_NAME                   = 4
Const CAPICOM_CERTIFICATE_FIND_EXTENSION                       = 5
Const CAPICOM_CERTIFICATE_FIND_EXTENDED_PROPERTY               = 6
Const CAPICOM_CERTIFICATE_FIND_APPLICATION_POLICY              = 7
Const CAPICOM_CERTIFICATE_FIND_CERTIFICATE_POLICY              = 8
Const CAPICOM_CERTIFICATE_FIND_TIME_VALID                      = 9
Const CAPICOM_CERTIFICATE_FIND_TIME_NOT_YET_VALID              = 10
Const CAPICOM_CERTIFICATE_FIND_TIME_EXPIRED                    = 11
Const CAPICOM_CERTIFICATE_FIND_KEY_USAGE                       = 12

Const CAPICOM_ENCODE_ANY = 4294967295 '0xffffffff 
Const CAPICOM_ENCODE_BASE64 = 0 
Const CAPICOM_ENCODE_BINARY = 1

' END CAPICOM Constants

' CSPs (CryptoGraphic Service Providers) names from wincrypt.h
' Default/Base         512 bits    default public key length     
Const MS_DEF_PROV                  = "Microsoft Base Cryptographic Provider v1.0"
' Enhanced            1,024 bits   default public key length     
Const MS_ENHANCED_PROV             = "Microsoft Enhanced Cryptographic Provider v1.0"
' Strong              1,024 bits   default public key length     
Const MS_STRONG_PROV               = "Microsoft Strong Cryptographic Provider"

Const CRYPT_EXPORTABLE             = 1
Const CRYPT_USER_PROTECTED         = 2

Const AT_KEYEXCHANGE               = 1
Const AT_SIGNATURE                 = 2

' from \nt\public\sdk\inc\certcli.h
Const CR_OUT_BASE64HEADER  =&H00000000
Const CR_OUT_BASE64        =&H00000001
Const CR_OUT_BINARY        =&H00000002
Const CR_OUT_CHAIN         =&H00000100
Const CR_OUT_CRLS          =&H00000200

'Disposition code ref: \nt\public\sdk\inc\certcli.h
Const CR_DISP_INCOMPLETE        =0
Const CR_DISP_ERROR             =1
Const CR_DISP_DENIED            =2
Const CR_DISP_ISSUED            =3
Const CR_DISP_ISSUED_OUT_OF_BAND=4
Const CR_DISP_UNDER_SUBMISSION  =5
Const CR_DISP_REVOKED           =6
Const no_disp=-1

Const CERT_SYSTEM_STORE_LOCAL_MACHINE = &H20000
Const 	CR_IN_BASE64HEADER= &H0
Const 	CR_IN_BASE64      = &H1
Const	CR_IN_BINARY	= &H2
Const	CR_IN_ENCODEANY	= &Hff
Const	CR_IN_ENCODEMASK= &Hff
Const	CR_IN_FORMATANY	= 0
Const	CR_IN_PKCS10	= &H100
Const	CR_IN_KEYGEN	= &H200
Const	CR_IN_PKCS7	= &H300
Const	CR_IN_CMC	= &H400
Const 	CR_IN_RPC 	= &H20000
Const 	CR_IN_FULLRESPONSE = &H40000



Const CR_PROP_CAEXCHGCERT = 15
Const PROPTYPE_BINARY     = 3

' from \nt\private\ca\include\certlib.h
Const GETCERT_CAXCHGCERT = 1	' == C/C++ TRUE value
Const GETCERT_CASIGCERT  = False

Const CRYPT_STRING_BASE64HEADER = &H0
Const CRYPT_STRING_BASE64       = &H1
Const CRYPT_STRING_BINARY       = &H2
Const CRYPT_STRING_BASE64REQUESTHEADER = &H3
Const CRYPT_STRING_HEX          = &H4

Const XECR_PKCS10_V2_0 = 1
Const XECR_PKCS7       = 2
Const XECR_CMC         = 3

Const NOT_CASE_SENSITIVE = 1

' Defined in MS code for Certificates MMC; 
' Supports certificate "renewal" (replace old cert with new)
Const XENROLL_RENEWAL_CERTIFICATE_PROP_ID = 33025

Const ERROR_NO_MORE_ITEMS = &h80070103
' Object doesn't support this property or method
Const ERROR_PROPERTY_UNSUPPORTED = &h1B6
Const ERROR_PARAMETER_REQUIRED = &h8000ffff

Dim bRetval
Dim args
Dim oCert
dim strCert64
Dim oCertStatus
Dim oStore
Dim nDisp 
Dim objXen
Dim objRequest
Dim i 
Dim strDN
Dim strUsageOID 
Dim strReq
Dim iIndex
Dim intProviderNum
Dim bCertValidCheck
Dim keyLengthMask
Dim storeName
Dim storeLoc
Dim strCACertificate
Dim strEKU
Dim tempErrNum
Dim tempErrDesc
Dim filteredCerts
Dim bfoundAllEKUs
Dim strConfig
Dim strMessage
Dim Certificate
Dim strCAaddr
Dim requestFlags
Dim WshShell
Dim signerCert
Dim bUseCMCRequest
Dim strExtraAttributes
Dim bDoingSigning
Dim objSigner
Dim objUtil
Dim strDecodedReq
Dim ObjSignedData
Dim strSignedRequest
Dim Subjects()
Dim signSubjects()
Dim certRequest
Dim CACert

' OIDs
Const OID_ServerAuth            = "1.3.6.1.5.5.7.3.1"
Const OID_ClientAuth            = "1.3.6.1.5.5.7.3.2"   
Const OID_CodeSigning           = "1.3.6.1.5.5.7.3.3"
Const OID_EmailProtection       = "1.3.6.1.5.5.7.3.4"
Const OID_encryptedFileSystemMS = "1.3.6.1.4.1.311.10.3.4"
Const OID_ENROLLMENT_AGENT      = "1.3.6.1.4.1.311.20.2.1"
Const OID_SmartCardLogon        = "1.3.6.1.4.1.311.20.2.2"
Const szOID_SUBJECT_ALT_NAME    = "2.5.29.7"
Const szOID_SUBJECT_ALT_NAME2   = "2.5.29.17"

' This is the list of OIDs based on CAPICOM_EKU enumeration
Dim CAPICOM_OIDs(5)
CAPICOM_OIDs(CAPICOM_EKU_OTHER)            = ""
CAPICOM_OIDs(CAPICOM_EKU_SERVER_AUTH)      = OID_ServerAuth
CAPICOM_OIDs(CAPICOM_EKU_CLIENT_AUTH)      = OID_ClientAuth
CAPICOM_OIDs(CAPICOM_EKU_CODE_SIGNING)     = OID_CodeSigning
CAPICOM_OIDs(CAPICOM_EKU_EMAIL_PROTECTION) = OID_EmailProtection
CAPICOM_OIDs(CAPICOM_EKU_SMARTCARD_LOGON)  = OID_SmartCardLogon

' see wincrypt.h
Const CERT_FRIENDLY_NAME_PROP_ID           = 11

' Command line parameters and defaults
' --------------------------------------------------------------
' "/?"
'    No associated variables
' "/ca"
Dim strCmdLineConfig       :  strCmdLineConfig = ""
' "/certtype"
Dim strCertType            :  strCertType      = DEFAULT_CERTIFICATE_TYPE
'   /v"
Dim bVerbose               :  bVerbose         = False
'  /force"
Dim bForce                 :  bForce           = False

'   /keyusage
Dim keyUsage               :  keyUsage            = AT_KEYEXCHANGE

'  /eku"
Dim intEKUs()
Dim strEKUs()
' These two aren't actually used, need some empty arrays to pass in.
Dim signintEKUs()
Dim signstrEKUs()
'  /apppolicy
Dim AppPolicies()
'  /certpolicy
Dim CertPolicies()
'  /signapppol
Dim SignAppPolicies()
'  /signcertpol
Dim SignCertPolicies()

'   /fn
Dim strCertFriendlyName    :  strCertFriendlyName = ""

'   /csp											(default to the advanced provider)
Dim strCSP                 :  strCSP              = MS_ENHANCED_PROV
'   /keyl                                           (0 key length means it will not try to set and just use CSP default)
Dim keyLength              :  keyLength           = 1024

'   /sign
Dim bSign                  :  bSign               = False
'   /strongkey
Dim bStrongKey             :  bStrongKey          = False
'   /onlyencipher
Dim bOnlyEncipher          :  bOnlyEncipher       = False
'   /export
Dim bExportable            :  bExportable       = False

'	/renew  -- default to renew 6 weeks (42 days) before expiration
Dim bRenew				   :  bRenew			= False
Dim numDaysBeforeExpirationToRenew : numDaysBeforeExpirationToRenew = 42

'	/archive				   
Dim bArchive			   :  bArchive			= False

	
nDisp = -1

' By default use PKCS10 request but certain operations require CMC
bUseCMCRequest = False
bDoingSigning  = False
strExtraAttributes = ""

Set WshShell = CreateObject("WScript.Shell")

' There is a problem -- earlier 500 error is not harvested
If Err.Number <> 0 Then
   WScript.Echo "Error creating WScript.Shell object"
End If

' Check to see if they need help
Set args = WScript.Arguments
if Err.Number <> 0 Then
    LogError Err.Number, Err.Description, "Accessing WScript.Arguments", bVerbose 
    Err.Clear
End if

if args.count = 1 then
    if args(0) = "/?" Then
      call printUsage
      WScript.Quit(0)
    end if
    if strcomp(LEFT(args(0), 2), "/h", NOT_CASE_SENSITIVE) = 0 then
      ' Matched /h* or /H* (/h, /help, /H, /HELP)
      call printUsage
      WScript.Quit(0)
    end if
end if


For i = 0 To args.Count - 1
    if strcomp(args(i), "/ca") = 0 then
         strCmdLineConfig = args(i+1) 
    elseif strcomp(args(i), "/certtype") = 0 then
         strCertType = args(i+1) 

    elseif strcomp(args(i), "/v") = 0 then
         bVerbose = True

    elseif strcomp(args(i), "/force") = 0 then
         bForce = True

    elseif strcomp(args(i), "/eku") = 0 then
       If IsReDimed(strEKUs) Then
         ' Make the arrays a little bigger to fit the new values.
         ReDim Preserve strEKUs(UBound(strEKUs)+1)
         ReDim Preserve intEKUs(UBound(intEKUs)+1)
       Else
         ' First time so don't preserve.
         ReDim strEKUs(0)
         ReDim intEKUs(0)
       End If
       strEKUs(UBound(strEKUs)) = args(i+1)
       ' change intEKU to 'other' to make sure we do the translateEKU later
       intEKUs(UBound(intEKUs)) = CAPICOM_EKU_OTHER
    elseif strcomp(args(i), "/apppolicy") = 0 then
       If IsReDimed(AppPolicies) Then
         ' Make the array a little bigger to fit the new value.
         ReDim Preserve AppPolicies(UBound(AppPolicies)+1)
       Else
         ' First time so don't preserve.
         ReDim AppPolicies(0)
       End If
    AppPolicies(UBound(AppPolicies)) = args(i+1)

    elseif strcomp(args(i), "/certpolicy") = 0 then
       If IsReDimed(CertPolicies) Then
         ' Make the array a little bigger to fit the new value.
         ReDim Preserve CertPolicies(UBound(CertPolicies)+1)
       Else
         ' First time so don't preserve.
         ReDim CertPolicies(0)
       End If
       CertPolicies(UBound(CertPolicies)) = args(i+1)

    elseif strcomp(args(i), "/signapppol") = 0 then
       If IsReDimed(SignAppPolicies) Then
         ' Make the array a little bigger to fit the new value.
         ReDim Preserve SignAppPolicies(UBound(SignAppPolicies)+1)
       Else
         ' First time so don't preserve.
         ReDim SignAppPolicies(0)
       End If
       SignAppPolicies(UBound(SignAppPolicies)) = args(i+1)

    elseif strcomp(args(i), "/signcertpol") = 0 then
       If IsReDimed(SignCertPolicies) Then
         ' Make the array a little bigger to fit the new value.
         ReDim Preserve SignCertPolicies(UBound(SignCertPolicies)+1)
       Else
         ' First time so don't preserve.
         ReDim SignCertPolicies(0)
       End If
       SignCertPolicies(UBound(SignCertPolicies)) = args(i+1)

    elseif strcomp(args(i), "/fn") = 0 then
         strCertFriendlyName = args(i+1)

    elseif strcomp(args(i), "/csp") = 0 then
         select case args(i+1)
             case "base"
                 strCSP = MS_DEF_PROV
             case "enhanced"
                 strCSP = MS_ENHANCED_PROV
             case "strong"
                 strCSP = MS_STRONG_PROV
             case else
                 WScript.Echo "Unrecognized csp, valid types are base, enhanced, strong" 
         end select

    elseif strcomp(args(i), "/keyl") = 0 then
        keyLength = Int(args(i+1))
        

    elseif strcomp(args(i), "/strongkey") = 0 then
        bStrongKey = True

    elseif strcomp(args(i), "/export") = 0 then
        bExportable = True
 
    ' Certificate renewal
    elseif strcomp(args(i), "/renew") = 0 then
        bRenew = True
	If i+1 < args.count Then
	   If IsNumeric(args(i+1)) Then
		numDaysBeforeExpirationToRenew = args(i+1) 
	   End If
	End If

    elseif strcomp(args(i), "/archive") = 0 then
        bArchive = True
        ' Key escrow requires CMC
        bUseCMCRequest = True
    end if

Next

' We at least need a certificate type to run
If strCertType = "" Then
	LogError ERROR_PARAMETER_REQUIRED, _
		"You must provide at least a certificate type to enroll. Use /certtype to specify it, e.g. CliAuth, Machine. ", _
		"Error Parsing command line parameters.", bVerbose
    WScript.Echo "You must provide at least a certificate type to enroll. Use /certtype to specify it, e.g. CliAuth, Machine. "
    WScript.Quit(1)
End If

Set oStore = CreateObject ("CAPICOM.Store")
if Err.Number <> 0 Then
    LogError Err.Number, Err.Description, "Creating CAPICOM.Store Object", bVerbose 
    WScript.Echo "CAPICOM is not installed. Exiting."
    WScript.Echo "You might need to run: regsvr32 capicom.dll"
    Err.Clear
    WScript.Quit(1)
End if


If UCase(strCertType) = UCase("Machine") Then
    storeLoc = CAPICOM_LOCAL_MACHINE_STORE
Else
    storeLoc = CAPICOM_CURRENT_USER_STORE
End if

' Look through all the certificates and check for ones using the EKUs and Policies specified.
' We don't want to add the new cert if we find all the specified EKUs and Policies unless /force is specified
' If we do find them all we will quit the program, unless /renew is specified.
' In that case, we will use the specified EKUs and Policies to locate the certificate for renewal
' NOTE: "Renewal" is really getting a new certificate and instructing the store to replace the old
' certificate with the new one.

Dim binaryStreamOfCertificateToRenew 
Dim bFoundCertificate
Dim daystoExpiration


If (bForce = False AND ( IsReDimed(intEKUs) OR IsReDimed(AppPolicies) OR IsReDimed(CertPolicies))) Then
	
    ' We aren't going to force it or we are renewing a certificate so now we start looking through the certs

    bFoundCertificate = FindCertificate (Certificate, AppPolicies, CertPolicies, Subjects, _
                         intEKUs, strEKUs, storeLoc, _
                         CAPICOM_MY_STORE, CAPICOM_STORE_OPEN_READ_ONLY)

    If bFoundCertificate = True Then
	daystoExpiration = DateDiff("d", Date, Certificate.ValidToDate)
		If (bRenew = True) AND (daystoExpiration < CInt(numDaysBeforeExpirationToRenew)) Then
			
			' We found the certificate we need to renew and it is in the renewal period
			' Export it in binary form so that we know which one to replace on renewal
			binaryStreamOfCertificateToRenew = Certificate.Export(CAPICOM_ENCODE_BINARY)		

			' Continue on to the enrollment
		Else

			If bRenew = True Then
				' We found the certificate but it is not in the renewal period
				LogError 0, "", "This machine has a certificate, but it is not within the renewal period", bVerbose 
				Wscript.Echo "This machine has a certificate, but it is not within the renewal period"
				WScript.Quit(0)
			Else	

				LogError 0, "", "This machine already has a certificate that matches the criteria specified", bVerbose 
				Wscript.Echo "This machine already has a certificate that matches the criteria specified"
				WScript.Quit(0)
			End If	
		End If

	Else
		If bRenew = True Then
		' We did not find the certificate to renew, so get a new one
			strMessage = "This machine does not have a certificate to renew that matches the criteria supplied."
			WScript.Echo strMessage
			LogError 0, "", strMessage, bVerbose 
			'WScript.Quit(0)
		End If
	End If
End If


' If we get to here: 
' (1) We are renewing an existing certificate that we were able to locate OR
' (2) We are forcing an enrollment OR
' (3) We did not find the certificate specified (and we are not renewing), 
'     so we are going to get a new certificate


'Build the distinguished name string. Dont need to set this as built on CA
strDN = ""

Set objXen = CreateObject("CEnroll.CEnroll.1")
If Err.Number <> 0 Then
  LogError Err.Number, Err.Description, "Creating CEnroll.CEnroll.1 object", bVerbose
  Err.Clear
End If


' Set the CSP (changes default key strength)
If strCSP <> MS_DEF_PROV Then
  ' Assume that only the non-default ones might be missing
  ' Check if it is one of the available CSPs
  intProviderNum = 0
  Do

If MY_DEBUG Then : On Error Resume Next : End If

      strProvider = objXen.enumProviders(intProviderNum, 0)
      If Err.Number <> 0 Then
          if Err.Number = ERROR_NO_MORE_ITEMS Then
            ' Hit the end of the list so Didn't find the provider
            Err.Clear
            WScript.Echo "Unable to locate the requested CSP:" & " [" & strCSP & "]" 
            WScript.Echo "CSP will be:" & objXen.ProviderName
          Else
            ' Some other error occurred, log it.
            LogError Err.Number, Err.Description, "Enumerating CSPs (enumProviders)", bVerbose
            Err.Clear
          End If
          Err.Clear
If MY_DEBUG Then : On Error Goto 0 : End If
          Exit Do
      Else
If MY_DEBUG Then : On Error Goto 0 : End If
          If strCSP = strProvider Then
            ' We found the requested provider
            objXen.ProviderName = strCSP
            If Err.Number <> 0 Then
              LogError Err.Number, Err.Description, "Setting the CSP provider", bVerbose
              Err.Clear
            End If
            If bVerbose Then
                WScript.Echo "CSP changed to:" & strCSP
            End If
            Exit Do
          End If
          intProviderNum = intProviderNum + 1
      End If
  Loop
End If

' Set the key length
If keyLength > 0 Then
    ' set the key size (the upper 16 bits of GenKeyFlags)
    ' The lower 16 bits contain the exportable flag (CRYPT_EXPORTABLE or CRYPT_USER_PROTECTED)
    keyLengthMask = keyLength * (2^16)
    objXen.GenKeyFlags = keyLengthMask
    if bVerbose Then
        WScript.Echo "Setting the key length to " & keyLength & " 0x" & hex(keyLengthMask)
    End If
End If

' Set Strong private key protection
If bStrongKey Then
    ' Turn on Strong private key protection
    ' This should prompt for a password every time the private key is used
    ' One reference indicated that this may only be supported by the enhanced provider
    objXen.GenKeyFlags = objXen.GenKeyFlags OR CRYPT_USER_PROTECTED
    if bVerbose Then
        WScript.Echo "Setting CRYPT_USER_PROTECTED 0x" & hex(objXen.GenKeyFlags)
    End If    
End If

' Set Exportable flag
If bExportable Then
    ' Turn on Exportable
    objXen.GenKeyFlags = objXen.GenKeyFlags OR CRYPT_EXPORTABLE
    if bVerbose Then
        WScript.Echo "Setting CRYPT_EXPORTABLE 0x" & hex(objXen.GenKeyFlags)
    End If    
End If


If bVerbose Then
  ' Retrieve the minimum and maximum key lengths.
  ' Display the results.
  WScript.Echo "For this provider the key lengths can be:" 
  WScript.Echo "  Exchange key Min: " & objXen.GetKeyLen(1, True) 
  If Err.Number <> 0 Then
    LogError Err.Number, Err.Description, "CEnroll.GetKeyLen", bVerbose
    Err.Clear
  End If
  WScript.Echo "    Max: " & objXen.GetKeyLen(False, 1)  
  WScript.Echo "  Signature key Min: " & objXen.GetKeyLen(1, False) 
  WScript.Echo "    Max: " & objXen.GetKeyLen(False, False)

  ' Display the available CSPs CryptoGraphic Service Providers
  Dim strProvider


If MY_DEBUG Then : On Error Resume Next : End If

  WScript.Echo "Available CSPs (CryptoGraphic Service Providers) Are:"
  intProviderNum = 0
  Do
      strProvider = objXen.enumProviders(intProviderNum, 0)
      If Err.Number <> 0 Then
          if Err.Number = ERROR_NO_MORE_ITEMS Then
            ' Hit the end of the list
            Err.Clear
            Exit Do
          Else
            ' Some other error occurred, log it.
            LogError Err.Number, Err.Description, "Enumerating CSPs (enumProviders)", bVerbose
            Err.Clear
          End If

      Else
          WScript.Echo "  " & intProviderNum & " [" & strProvider & "]" 
          intProviderNum = intProviderNum + 1
      End If
  Loop
  '
  WScript.Echo "Current CSP is:" & objXen.ProviderName
End If

If MY_DEBUG Then : On Error Goto 0 : End If

Set objRequest = CreateObject ("CertificateAuthority.Request")
If Err.Number <> 0 Then
  LogError Err.Number, Err.Description, "Creating CertificateAuthority.Request object", bVerbose
  Err.Clear
End If

'Change to the machine stores
if UCase(strCertType) = UCase("Machine") Then
    objXen.MyStoreFlags   = CERT_SYSTEM_STORE_LOCAL_MACHINE
    objXen.CAStoreFlags   = CERT_SYSTEM_STORE_LOCAL_MACHINE
    objXen.RootStoreFlags = CERT_SYSTEM_STORE_LOCAL_MACHINE
    strConfig = strMachineConfig
Else
    ' MyStoreFlags   defaults to CERT_SYSTEM_STORE_CURRENT_USER
    ' CAStoreFlags   defaults to CERT_SYSTEM_STORE_CURRENT_USER
    ' RootStoreFlags defaults to CERT_SYSTEM_STORE_CURRENT_USER
    strConfig = strUserConfig
End If
If Len(strCmdLineConfig) > 0 Then
    ' Override the value with the one provided on the command line
    strConfig = strCmdLineConfig
End If

' add a cert type to the request
objXen.addCertTypeToRequest strCertType
If Err.Number <> 0 Then
  LogError Err.Number, Err.Description, "Doing addCertTypeToRequest", bVerbose
  Err.Clear
End If
If bVerbose Then
  WScript.Echo "Doing addCertTypeToRequest value = [" & strCertType & "]"
End If

' Add a friendly name to the certificate request
If Len(strCertFriendlyName) > 0 Then
  If bVerbose Then
    WScript.Echo "Adding a friendly Name = [" & strCertFriendlyName & "]"
  End If
  ' Docs say parameter 2 must be 0 but MS code uses XECP_STRING_PROPERTY=1
  objXen.addBlobPropertyToCertificate CERT_FRIENDLY_NAME_PROP_ID, 0, strCertFriendlyName
  If Err.Number <> 0 Then
    LogError Err.Number, Err.Description, "Doing addBlobPropertyToCertificate of Friendly Name", bVerbose
    Err.Clear
  End If
End If

' Set the Key Usage on the key to AT_KEYEXCHANGE or AT_SIGNATURE 
If bVerbose Then
  select case keyUsage
    case AT_SIGNATURE
        WScript.Echo "Setting keyUsage/KeySpec to AT_SIGNATURE"
    case AT_KEYEXCHANGE
        WScript.Echo "Setting keyUsage/KeySpec to AT_KEYEXCHANGE"
    case else
        WScript.Echo "Setting keyUsage/KeySpec to [" & keyUsage & "]" 
  end select
End If
objXen.KeySpec = keyUsage
If Err.Number <> 0 Then
  LogError Err.Number, Err.Description, "Doing KeySpec", bVerbose
  Err.Clear
End If

' Set the Encipherment limit on the exchange key
If bVerbose Then
  WScript.Echo "Doing LimitExchangeKeyToEncipherment value = [" & bOnlyEncipher & "]"
End If
objXen.LimitExchangeKeyToEncipherment = bOnlyEncipher
If Err.Number <> 0 Then
  LogError Err.Number, Err.Description, "Doing LimitExchangeKeyToEncipherment", bVerbose
  Err.Clear
End If


' Take the command line parameter EKU(s) and create a comma separated list 
' to send to createPKCS10
strUsageOID = ""
If IsReDimed(strEKUs) Then
  For each strEKU in strEKUs
      if Len (strUsageOID) > 0 Then
          ' createPKCS10 takes a comma separated list of OIDs for the usage.
          strUsageOID = strUsageOID & "," & strEKU
      Else
          strUsageOID = strEKU
      End If
  Next
Else
    ' We don't have any so default to OID_ClientAuth
     strUsageOID = OID_ClientAuth
End If

If bArchive Then
	' Retrieve the CA's key archival certificate
	Set CertRequest = CreateObject( "CertificateAuthority.Request" )
	
	If bVerbose Then
		WScript.Echo "Retrieving CA key archival certificate"
	End If

	' First argument to GetCACertificate is fExchangeCertificate, a boolean
	' Win2000 only supported arguement is FALSE
	' Win2003 TRUE (1) will return CA cert to encrypt requests to CA
	CACert = CertRequest.GetCACertificate( 1, strConfig, CR_OUT_BASE64 )

	If Err.Number <> 0 Then
		LogError Err.Number, Err.Description, "Doing GetCACertificate", bVerbose
		Err.Clear
	End If

	' Add the CA's certificate to the request
	objXen.PrivateKeyArchiveCertificate = CACert
End If

If (bRenew AND bFoundCertificate) Then
	' Add the Renewal Certificate Property
	' When we install the cert the old certificate will be deleted
	objXen.addBlobPropertyToCertificate XENROLL_RENEWAL_CERTIFICATE_PROP_ID, 0, binaryStreamOfCertificateToRenew

	' Use this to generate new private key pair
	objXen.UseExistingKeySet = False
End If

' Sign the request with a certificate with a certificate that contains the supplied policies
' If either has been reDimmed then they contain values from command line
If IsReDimed(SignAppPolicies) OR IsReDimed(SignCertPolicies) Then
    ' Find a certificate with the appropriate policies to use to sign the request
    ' The certificate must be on the machine
    If FindCertificate (Certificate, signAppPolicies, signCertPolicies, signSubjects, _
                         signintEKUs, signstrEKUs, storeLoc, _
                         CAPICOM_MY_STORE, CAPICOM_STORE_OPEN_READ_ONLY) _
    Then
        ' turns out Signercertificate doesn't function, use a different method
'        ' Export the certificate we found and use it as the signer certificate in the request.
'        ' Default of export should be CAPICOM_ENCODE_BASE64
'        signerCert = Certificate.Export
'        If Err.Number <> 0 Then
'          LogError Err.Number, Err.Description, "Exporting signer certificate", bVerbose
'          Err.Clear
'        End If
'        objXen.signercertificate = signerCert
'        If Err.Number <> 0 Then
'          LogError Err.Number, Err.Description, "Assigning certificate to signercertificate", bVerbose
'          Err.Clear
'        End If
'
'        ' Signed requests needs CMC request type not PKCS10
'        bUseCMCRequest = True
'
'        If bVerbose Then
'            WScript.Echo "Setting signer certificate to this certificate (also shown above)"
'            WScript.Echo "<Start Signer Cert>"
'            WScript.Echo signerCert
'            WScript.Echo "<End Signer Cert>"
'        End If

        If bVerbose Then
            WScript.Echo "Using the certificate that was just displayed to sign the certificate request"
        End If

        Set objSigner = CreateObject("CAPICOM.Signer.2")
        If Err.Number <> 0 Then
          LogError Err.Number, Err.Description, "Creating CAPICOM.Signer.2 object", bVerbose
          Err.Clear
        End If

        objSigner.Certificate = Certificate
        If Err.Number <> 0 Then
          LogError Err.Number, Err.Description, "Assigning signing certificate to signer object", bVerbose
          Err.Clear
        End If

        bDoingSigning = True

    Else
        ' We require a cert to sign and didn't find it so quit
        strMessage = "Unable to find signing certificate with required policies."
        WScript.Echo "Unable to find signing certificate with required policies."
        ' *********** WE DIDN'T FIND A CERT WITH NEEDED Policy TO SIGN SO QUIT ********************
        LogError 0, "", strMessage, bVerbose
        WScript.Quit(0)
    End If
End If

' Now create the request string and submit it.
bRetval = False
If strcomp(strConfig, "") <> 0 Then
    ' Only do this if we have a CA name

    ' Create the request string
    If bUseCMCRequest Then
        ' When archiving the private key need to use CMC, we get back a string in BASE64_HEADER format
        ' Also used for signed requests.
        strReq = objXen.CreateRequest(XECR_CMC, strDN, strUsageOID)
        If Err.Number <> 0 Then
            WScript.Echo "Error during CreateRequest: " & Err.Description
            LogError Err.Number, Err.Description, "Doing CreateRequest", bVerbose
            Err.Clear
        End If
        requestFlags = CR_IN_ENCODEANY Or CR_IN_FORMATANY Or CR_IN_RPC Or CR_IN_FULLRESPONSE
    Else
        strReq = objXen.createPKCS10(strDN, strUsageOID)
        If Err.Number <> 0 Then
            WScript.Echo "Error during createPKCS10: " & Err.Description
            LogError Err.Number, Err.Description, "Doing createPKCS10", bVerbose
            Err.Clear
        End If
        requestFlags = CR_IN_BASE64 Or CR_IN_PKCS10

        If bDoingSigning Then

            requestFlags = CR_IN_BASE64 Or CR_IN_FORMATANY

            ' Create a request (using xenroll) for the requisite template in PKCS10 format (CreatePKCS10).

            ' Decode the PKCS10 Request
            Set objUtil = CreateObject("CAPICOM.Utilities.1")
            If Err.Number <> 0 Then
            LogError Err.Number, Err.Description, "Creating Object CAPICOM.Utilities.1", bVerbose
            Err.Clear
            End If

            strDecodedReq = objUtil.Base64Decode(strReq)
            If Err.Number <> 0 Then
            LogError Err.Number, Err.Description, "Doing Base64Decode", bVerbose
            Err.Clear
            End If

            ' Create the SignedData object and assign its inner content as the decoded PKCS10 request
            Set ObjSignedData = CreateObject("CAPICOM.SignedData.1")
            If Err.Number <> 0 Then
            LogError Err.Number, Err.Description, "Creating Object CAPICOM.SignedData.1", bVerbose
            Err.Clear
            End If

            ObjSignedData.Content = strDecodedReq

            ' Sign this content with the signing certificate and save the resultant signedContent back to our request string
            strSignedRequest = ObjSignedData.Sign(ObjSigner,0)
            If Err.Number <> 0 Then
            LogError Err.Number, Err.Description, "Signing data", bVerbose
            Err.Clear
            End If

            strReq = strSignedRequest

        End if ' bDoingSigning

    End If

    if bVerbose Then
        WScript.Echo "Certificate Request base64: length =" & Cstr(Len(strReq))
        WScript.Echo ""
        WScript.Echo strReq
    End if

    bRetval = SubmitRequest ( strReq, strConfig, requestFlags, strExtraAttributes )

End If

if bRetval = False Then
    strMessage  = "Enrollment failed."
    WScript.Echo strMessage
    LogError 0, "", strMessage, bVerbose
    WScript.Quit (1)
End if

' Accept the response from the CA
If bUseCMCRequest Then
    objXen.acceptResponse ( strCert64 ) 
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Doing acceptResponse", bVerbose
      Err.Clear
      bRetval = False
    End If
Else
    objXen.acceptPKCS7 ( strCert64 ) 
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Doing acceptPKCS7", bVerbose
      Err.Clear
      bRetval = False
    End If
End If 

if bVerbose Then
    WScript.Echo "Certificate:"
    'WScript.Echo strCert64
    Dim oCert2
    Set oCert2 = CreateObject ( "CAPICOM.Certificate" )
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Creating CAPICOM.Certificate object", bVerbose
      Err.Clear
    End If
    oCert2.Import objXen.getCertFromPKCS7( strCert64 )
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Doing Import of a getCertFromPKCS7", bVerbose
      Err.Clear
    End If
    PrintCert oCert2, True
End if

if bRetval = False Then
    WScript.Echo "Failed"
Else
    WScript.Echo "Success"
End If

' Clean up
Set args = Nothing
Set oStore = Nothing
Set oCert2 = Nothing
Set oCert = Nothing
Set oCertStatus = Nothing
Set objXen = Nothing
Set objRequest = Nothing
Set WshShell = Nothing

'******************************************************************************
'
' Function  : FindCertificate
'
' Synopsis  : 
'
' Parameter : Certificate - The found certificate collection.
'
' Return    : Boolean, certificate was found, true/false
'
'******************************************************************************

Function FindCertificate (Certificate, AppPolicies, CertPolicies, Subjects, intEKUs, strEKUs, storeLoc, storeName, storeMode)

If Not MY_DEBUG Then : On Error Resume Next : End If

    FindCertificate = False 

    oStore.Open storeLoc, storeName, storeMode
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Opening Cert store " & _
        storeLoc & "," & storeName & "," & storeMode, bVerbose
      Err.Clear
    End If

    ' Filter it down by policies or other filters applied so
    Set filteredCerts = FilterCertificates(oStore.Certificates, AppPolicies, CertPolicies, Subjects)

    if bVerbose Then 
        WScript.Echo "Found " & filteredCerts.Count & " certs with AppPolicies, CertPolicies (still going to check EKUs)"
    end if

    If filteredCerts.Count > 0 Then 
      IF IsReDimed(intEKUs) Then
        ' The filter has at least one cert and we have one or more EKUs specified

        ' Make sure all our EKUs are in set up correctly
        For iIndex = LBound(intEKUs) To UBound(intEKUs) 
          if intEKUs(iIndex) <> CAPICOM_EKU_CLIENT_AUTH Then
            ' Find the EKU Enum value that corresponds to our command line EKU string
            intEKUs(iIndex) = translateEKU (strEKUs(iIndex)) 
          End If
        Next

        ' Now look through each cert to see if has all the EKUs specified
        For Each oCert in filteredCerts
          Set oCertStatus = oCert.IsValid
          If Err.Number <> 0 Then
            LogError Err.Number, Err.Description, "Checking if certificate IsValid", bVerbose
            Err.Clear
          End If

          bfoundAllEKUs = True
          For iIndex = LBound(intEKUs) To UBound(intEKUs)

              oCertStatus.EKU.Name = intEKUs(iIndex)
              If Not HasEKU ( oCertStatus, intEKUs(iIndex), strEKUs(iIndex) ) Then
                  ' This cert is missing one of the EKUs
                  bfoundAllEKUs = False
              End If 

          Next ' All EKUs
        
          if bfoundAllEKUs Then
              if bVerbose Then 
                  WScript.Echo "--- Existing Cert contains all EKUs and Policies ----------------"
                  PrintCert  oCert , bVerbose
              end if
              WScript.Echo "This machine already has a certificate for specified EKUs and Policies."

              ' *********** WE FOUND ALL THE EKUs and Policies SO WE ARE EXITING ********************
              Set Certificate = oCert
              FindCertificate = True
              Exit Function
          Else
            ' We will continue on and check more certs.
          End If

        Next ' All certificates
      Else
        ' No EKUs were specified and we have at least one certificate came out of our filter
        ' If at least one of the remaining certificates is valid then we should quit
        bCertValidCheck = False
        For Each oCert in filteredCerts
          bCertValidCheck = oCert.IsValid
          If Err.Number <> 0 Then
            LogError Err.Number, Err.Description, "Checking if certificate IsValid", bVerbose
            Err.Clear
          End If
          If bCertValidCheck Then
              if bVerbose Then 
                  WScript.Echo "--- Existing Cert contains all EKUs and Policies ----------------"
                  PrintCert  oCert , bVerbose
              end if
              ' *********** WE FOUND ALL THE Policies SO WE ARE EXITING ********************
              Set Certificate = oCert
              FindCertificate = True
              Exit Function
           End If ' bCertValidCheck
        Next ' All certificates

      End If ' IsReDimed(intEKUs)
    End If ' Filtered.Count

End Function ' End FindCertificate

'******************************************************************************
'
' Function  : FilterCertificates
'
' Synopsis  : Filter the set of certificates based on the filtering options.
'
' Parameter : Certificates - The certificates collection to be filtered.
'
' Return    : The filtered certificates collection.
'
'
'******************************************************************************

Function FilterCertificates (Certificates, AppPolicies, CertPolicies, Subjects)
If Not MY_DEBUG Then : On Error Resume Next : End If

   Dim iIndex
   Dim Filtered
   Set Filtered = Certificates
   
   If Filtered.Count > 0 AND IsReDimed(AppPolicies) Then
      For iIndex = LBound(AppPolicies) To UBound(AppPolicies) 
         Set Filtered = Filtered.Find(CAPICOM_CERTIFICATE_FIND_APPLICATION_POLICY, AppPolicies(iIndex))
         If Err.Number <> 0 Then
           LogError Err.Number, Err.Description, "Filtering Certificates on Policy", bVerbose
           Err.Clear
         End If
      Next
   End If

   If Filtered.Count > 0 AND IsReDimed(CertPolicies) Then
      For iIndex = LBound(CertPolicies) To UBound(CertPolicies) 
         Set Filtered = Filtered.Find(CAPICOM_CERTIFICATE_FIND_CERTIFICATE_POLICY, CertPolicies(iIndex))
         If Err.Number <> 0 Then
           LogError Err.Number, Err.Description, "Filtering Certificates on Certificate Policy", bVerbose
           Err.Clear
         End If
      Next
   End If

   If Filtered.Count > 0 AND IsReDimed(Subjects) Then
      For iIndex = LBound(Subjects) To UBound(Subjects) 
         Set Filtered = Filtered.Find(CAPICOM_CERTIFICATE_FIND_SUBJECT_NAME, Subjects(iIndex))
         If Err.Number <> 0 Then
           LogError Err.Number, Err.Description, "Filtering Certificates on Subject", bVerbose
           Err.Clear
         End If
      Next
   End If
   
   Set FilterCertificates = Filtered

End Function ' End FilterCertificates

'******************************************************************************
' Function  : translateEKU
'
' Synopsis  : Find a EKU Enum value that matches the string EKU OID we are provided.
'
' Parameter : EKUOIDString - String EKU OID to try to match to a built in CAPICOM OID.
'
' Return    : EKUEnum      - A number corresponding to a EKU Enum value.
'
' Notes :     Uses global CAPICOM_OIDs array
'******************************************************************************
Function translateEKU (EKUOIDString)
If Not MY_DEBUG Then : On Error Resume Next : End If
  Dim OIDnum

  ' Default to other
  translateEKU = CAPICOM_EKU_OTHER

  ' The OIDs in the CAPUCOM_OIDs are 'special', check if the one we were given is one 
  '  of them. Otherwise we will use CAPICOM_EKU_OTHER.
  OIDnum = Lbound(CAPICOM_OIDs)
  Do While OIDnum =< Ubound(CAPICOM_OIDs)
       If OIDnum <> CAPICOM_EKU_OTHER Then
         If EKUOIDString = CAPICOM_OIDs(OIDnum) Then
           ' The EKU OID string matched one of our CAPICOM OIDs so return that EKUEnum
           translateEKU = OIDnum
           ' This will cause us to exit the loop
           OIDnum = Ubound(CAPICOM_OIDs)
         End If
       End If
       OIDnum=OIDnum + 1 
  Loop ' End Do While


End Function ' End translateEKU

'******************************************************************************
Function HasEKU ( cs , EKUEnum, EKUOIDString )
If Not MY_DEBUG Then : On Error Resume Next : End If
    Dim retval

    retval = False

    if EKUEnum = CAPICOM_EKU_OTHER Then
        cs.EKU.Name = EKUEnum
        cs.EKU.OID = EKUOIDString
    Else
        cs.EKU.Name = EKUEnum
    End If

    retval = cs.Result
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Checking Result in Certificate Status", bVerbose
      Err.Clear
    End If

    HasEKU = retval
End Function

'******************************************************************************
Function SubmitRequest (strReq , strConfig, requestFlags, strExtraAttributes)
If Not MY_DEBUG Then : On Error Resume Next : End If
    Dim retval
    Dim nDisp
    Dim strErrorMsg
    Dim errStatus
    Dim certRequestId

    retval = False
    nDisp = -1
    
    WScript.Echo "Submitting to " & strConfig

    nDisp = objRequest.Submit(requestFlags, strReq, strExtraAttributes, strConfig)
    if Err.Number <> 0 Then
        WScript.Echo "Error during submit: " & Err.Description
        LogError Err.Number, Err.Description, "Doing submit", bVerbose
        Err.Clear
    End If
    certRequestId = objRequest.GetRequestId()
    if Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Doing GetRequestId", bVerbose
      Err.Clear
    Else
      If bVerbose Then
        WScript.Echo "Certificate request number = " & certRequestId
      End If
    End If

    
    if nDisp = CR_DISP_ISSUED Then
        strCert64 = objRequest.GetCertificate ( CR_OUT_BASE64 Or CR_OUT_CHAIN )
        if Err.Number <> 0 Then
          LogError Err.Number, Err.Description, "Doing GetCertificate", bVerbose
          Err.Clear
        End If
        retval = True
    Else
        select case nDisp
            case CR_DISP_INCOMPLETE
                WScript.Echo "submit result = CR_DISP_INCOMPLETE " & CStr(nDisp)
            case CR_DISP_ERROR
                WScript.Echo "submit result = CR_DISP_ERROR " & CStr(nDisp)
            case CR_DISP_DENIED
                WScript.Echo "submit result = CR_DISP_DENIED " & CStr(nDisp)
            case CR_DISP_ISSUED_OUT_OF_BAND
                WScript.Echo "submit result = CR_DISP_ISSUED_OUT_OF_BAND " & CStr(nDisp)
            case CR_DISP_UNDER_SUBMISSION
                ' Don't treat as an error o don't display unless verbose.
                If bVerbose Then
                  WScript.Echo "submit result = CR_DISP_UNDER_SUBMISSION " & CStr(nDisp)
                End If
            case CR_DISP_REVOKED
                WScript.Echo "submit result = CR_DISP_REVOKED " & CStr(nDisp)
            case else
                WScript.Echo "submit result = " & CStr(nDisp)
        end select 
        if nDisp = CR_DISP_UNDER_SUBMISSION Then
            strErrorMsg = "The certificate request was pended, request ID # " & certRequestId
            WScript.Echo strErrorMsg
        elseif nDisp <> -1 Then
            errStatus = objRequest.GetLastStatus
            if Err.Number <> 0 Then
              LogError Err.Number, Err.Description, "Doing GetLastStatus", bVerbose
              Err.Clear
            End If

If MY_DEBUG Then : On Error Resume Next : End If

            WScript.Echo "Request Disposition:" & objRequest.GetDispositionMessage()
            if Err.Number <> 0 Then
              LogError Err.Number, Err.Description, "Doing GetDispositionMessage ( 0x" & Hex(errStatus) & " )", bVerbose
              Err.Clear
            End If

            strErrorMsg = "Request Error #( 0x" & Hex(errStatus) & " )"
            strErrorMsg = "Reason: " & objRequest.GetErrorMessageText(errStatus, 0 ) &  "( 0x" & Hex(errStatus) & " )"
            if Err.Number <> 0 Then
              if Err.Number = ERROR_property_unsupported Then
                ' Do nothing, not included in W2K client right now, need ICertRequest2 interface
              Else 
                ' This might not be supported by the client
                LogError Err.Number, Err.Description, "Doing GetErrorMessageText ( 0x" & Hex(errStatus) & " )", bVerbose
                Err.Clear
              End If
            End If

If MY_DEBUG Then : On Error Goto 0 : End If

			WScript.Echo strErrorMsg
        end if
    End if
    SubmitRequest = retval
End Function

'******************************************************************************
Sub PrintCert (cert, Verbose)
If Not MY_DEBUG Then : On Error Resume Next : End If
    Dim theEKUs
    Dim i
    Dim streku
    Dim iIndex

    Dim KeySpecStrings(2)
    KeySpecStrings(0) = "Unknown"
    KeySpecStrings(1) = "Exchange"
    KeySpecStrings(2) = "Signature"

    Dim ProviderTypes(24)
    ProviderTypes(0)  = "Unknown"
    ProviderTypes(1)  = "PROV_RSA_FULL"
    ProviderTypes(2)  = "PROV_RSA_SIG"
    ProviderTypes(3)  = "PROV_DSS"
    ProviderTypes(4)  = "PROV_FORTEZZA"
    ProviderTypes(5)  = "PROV_MS_EXCHANGE"
    ProviderTypes(6)  = "PROV_SSL"
    ProviderTypes(7)  = "PROV_STT_MER"
    ProviderTypes(8)  = "PROV_STT_ACQ"
    ProviderTypes(9)  = "PROV_STT_BRND"
    ProviderTypes(10) = "PROV_STT_ROOT"
    ProviderTypes(11) = "PROV_STT_ISS"
    ProviderTypes(12) = "PROV_RSA_SCHANNEL"
    ProviderTypes(13) = "PROV_DSS_DH"
    ProviderTypes(14) = "PROV_EC_ECDSA_SIG"
    ProviderTypes(15) = "PROV_EC_ECNRA_SIG"
    ProviderTypes(16) = "PROV_EC_ECDSA_FULL"
    ProviderTypes(17) = "PROV_EC_ECNRA_FULL"
    ProviderTypes(18) = "PROV_DH_SCHANNEL"
    ProviderTypes(20) = "PROV_SPYRUS_LYNKS"
    ProviderTypes(21) = "PROV_RNG"
    ProviderTypes(22) = "PROV_INTEL_SEC"
    ProviderTypes(23) = "PROV_REPLACE_OWF"
    ProviderTypes(24) = "PROV_RSA_AES"

    if Verbose Then
        WScript.Echo "  Subject Name: " & cert.SubjectName
        WScript.Echo "  Email name  = " & cert.GetInfo(CAPICOM_CERT_INFO_SUBJECT_EMAIL_NAME)
        WScript.Echo "  Issuer Name: " & cert.IssuerName
        WScript.Echo "  SHA-1 Thumbprint: " & cert.Thumbprint
        WScript.Echo "  Serial Number: " & cert.SerialNumber
        WScript.Echo "  Version: " & cert.Version
        WScript.Echo "  Valid From: " & cert.ValidFromDate
        WScript.Echo "  Valid To: " & cert.ValidToDate
        WScript.Echo "  IsValid: " & cert.IsValid()
    Else
        WScript.Echo "  Subject Name: " & cert.SubjectName
        WScript.Echo "  SHA-1 Thumbprint: " & cert.Thumbprint
        WScript.Echo "  Valid To: " & cert.ValidToDate
    End If
    if Verbose Then

       If cert.BasicConstraints.IsPresent Then
          Wscript.Stdout.Writeline "Basic Constraints:" 
          Wscript.Stdout.Writeline "  Critical          = " & cert.BasicConstraints.IsCritical
          Wscript.Stdout.Writeline "  CA                = " & cert.BasicConstraints.IsCertificateAuthority
          Wscript.stdout.Write     "  PathLenConstraint = "
          If cert.BasicConstraints.IsPathLenConstraintPresent Then
             Wscript.Stdout.Writeline CStr(cert.BasicConstraints.PathLenConstraint)
          Else
             Wscript.Stdout.Writeline "Not present."
          End If
       Else
          Wscript.Stdout.Writeline "Basic Constraints: Not present."
       End If

       If cert.KeyUsage.IsPresent Then
          Wscript.Stdout.Writeline "Key Usage:"
          Wscript.Stdout.Writeline "  Critical                  = " & cert.KeyUsage.IsCritical
          Wscript.Stdout.Writeline "  IsDigitalSignatureEnabled = " & cert.KeyUsage.IsDigitalSignatureEnabled 
          Wscript.Stdout.Writeline "  IsNonRepudiationEnabled   = " & cert.KeyUsage.IsNonRepudiationEnabled
          Wscript.Stdout.Writeline "  IsKeyEnciphermentEnabled  = " & cert.KeyUsage.IsKeyEnciphermentEnabled
          Wscript.Stdout.Writeline "  IsDataEnciphermentEnabled = " & cert.KeyUsage.IsDataEnciphermentEnabled
          Wscript.Stdout.Writeline "  IsKeyAgreementEnabled     = " & cert.KeyUsage.IsKeyAgreementEnabled
          Wscript.Stdout.Writeline "  IsKeyCertSignEnabled      = " & cert.KeyUsage.IsKeyCertSignEnabled
          Wscript.Stdout.Writeline "  IsCRLSignEnabled          = " & cert.KeyUsage.IsCRLSignEnabled
          Wscript.Stdout.Writeline "  IsEncipherOnlyEnabled     = " & cert.KeyUsage.IsEncipherOnlyEnabled
          Wscript.Stdout.Writeline "  IsDecipherOnlyEnabled     = " & cert.KeyUsage.IsDecipherOnlyEnabled
       Else
          Wscript.Stdout.Writeline "Key Usage: Not present."
       End If

       Wscript.Stdout.Writeline "Public Key:"
       Wscript.Stdout.Writeline "  Algorithm  = " & cert.PublicKey.Algorithm.FriendlyName & "(" & cert.PublicKey.Algorithm.Value & ")"
       Wscript.Stdout.Writeline "  Length     = " & CStr(cert.PublicKey.Length) & " bits"
       Wscript.Stdout.Writeline "  Key blob   = " & cert.PublicKey.EncodedKey.Value
       Wscript.Stdout.Writeline "  Parameters = " & cert.PublicKey.EncodedParameters.Value

       If cert.HasPrivateKey Then
          Wscript.Stdout.Writeline "Private Key:"
          Wscript.Stdout.Writeline "  Container name   = " & cert.PrivateKey.ContainerName
          ' Don't display unique container name for hardware token because it may cause UI to be displayed.
'          If StoreLocation = CAPICOM_SMART_CARD_USER_STORE OR Not cert.PrivateKey.IsHardwareDevice Then
'             Wscript.Stdout.Writeline "  Unique name      = " & cert.PrivateKey.UniqueContainerName
'          End If
          Wscript.Stdout.Writeline "  Provider name    = " & cert.PrivateKey.ProviderName
          Wscript.StdOut.Write     "  Provider type    = " 
          If cert.PrivateKey.ProviderType > UBound(ProviderTypes) Then
             Wscript.Stdout.Writeline ProviderTypes(0) & " (" & CStr(cert.PrivateKey.ProviderType) & ")"
          Else
             Wscript.Stdout.Writeline ProviderTypes(cert.PrivateKey.ProviderType) & " (" & CStr(cert.PrivateKey.ProviderType) & ")"
          End If
          Wscript.StdOut.Write     "  Key spec         = " 
          If cert.PrivateKey.KeySpec > UBound(KeySpecStrings) Then
             Wscript.Stdout.Writeline KeySpecStrings(0) & " (" & CStr(cert.PrivateKey.KeySpec) & ")"
          Else
             Wscript.Stdout.Writeline KeySpecStrings(cert.PrivateKey.KeySpec) & " (" & CStr(cert.PrivateKey.KeySpec) & ")"
          End If
          Wscript.Stdout.Writeline "  Accessible       = " & cert.PrivateKey.IsAccessible
          Wscript.Stdout.Writeline "  Protected        = " & cert.PrivateKey.IsProtected
          Wscript.Stdout.Writeline "  Exportable       = " & cert.PrivateKey.IsExportable
          Wscript.Stdout.Writeline "  Removable        = " & cert.PrivateKey.IsRemovable
          Wscript.Stdout.Writeline "  Machine keyset   = " & cert.PrivateKey.IsMachineKeyset
          Wscript.Stdout.Writeline "  Hardware storage = " & cert.PrivateKey.IsHardwareDevice
       Else
          Wscript.Stdout.Writeline "Private Key: Not found."
       End If

      iIndex = 0
      Dim Extension
      For Each Extension In cert.Extensions
         iIndex = iIndex + 1
         Wscript.Stdout.Writeline "Extension #" & CStr(iIndex) & ": " & Extension.OID.FriendlyName & "(" & Extension.OID.Value & ")"
         Wscript.Stdout.Writeline "  " & Extension.EncodedData.Format(CAPICOM_ENCODED_DATA_FORMAT_MULTI_LINES)

         If Not Extension.EncodedData.Decoder Is Nothing Then
            Select Case Extension.OID.Value
            Case CAPICOM_CERT_POLICIES_OID
               Dim CertPolicies
               Set CertPolicies = Extension.EncodedData.Decoder
               Wscript.Stdout.Writeline "Decoded Certificate Policies: " & CStr(CertPolicies.Count) & " PolicyInformation(s)"
               
               Dim pIndex : pIndex = 0
               Dim PolicyInformation
               For Each PolicyInformation In CertPolicies
                  pIndex = pIndex + 1
                  Wscript.Stdout.Writeline "  PolicyInformation #" & CStr(pIndex) & ": " & CStr(PolicyInformation.Qualifiers.Count) & " Qualifier(s)"
                  Wscript.Stdout.Writeline "    OID = " & PolicyInformation.OID.FriendlyName & "(" & PolicyInformation.OID.Value & ")"
                  Dim qIndex : qIndex = 0
                  Dim Qualifier
                  For Each Qualifier In PolicyInformation.Qualifiers
                     qIndex = qIndex + 1
                     Wscript.Stdout.Writeline "    Qualifier #" & CStr(qIndex) & ":"
                     Wscript.Stdout.Writeline "      OID               = " & Qualifier.OID.FriendlyName & "(" & Qualifier.OID.Value & ")"
                     Wscript.Stdout.Writeline "      CPS URI           = " & Qualifier.CPSPointer
                     Wscript.Stdout.Writeline "      Organization name = " & Qualifier.OrganizationName
                     Wscript.Stdout.Write     "      Notice number(s)  = "
                     If Not Qualifier.NoticeNumbers Is Nothing Then
                        Dim nIndex
                        For nIndex = 1 to Qualifier.NoticeNumbers.Count
                           If nIndex > 1 Then
                              Wscript.Stdout.Write ", "
                           End If
                           Wscript.Stdout.Write CStr(Qualifier.NoticeNumbers.Item(nIndex))
                        Next
                     End If
                     Wscript.Stdout.Writeline "      Explicit text     = " & Qualifier.ExplicitText
                  Next
               Next

            Case Else
               ' We don't have the decoder, so can't do much.
            End Select
         End If
      Next
      If iIndex = 0 Then
         Wscript.Stdout.Writeline "Extension: None."
      End If
      iIndex = 0
      Dim ExtendedProperty
      For Each ExtendedProperty In cert.ExtendedProperties
         iIndex = iIndex + 1
         Wscript.Stdout.Writeline "Property #" & CStr(iIndex) & " (ID = " & ExtendedProperty.PropID & "):" 
         Wscript.Stdout.Writeline "  " & ExtendedProperty.Value
      Next
      If iIndex = 0 Then
         Wscript.Stdout.Writeline "Property: None."
      End If

    End If ' If Verbose

    If cert.ExtendedKeyUsage.IsPresent Then
      Set theEKUs = cert.ExtendedKeyUsage.EKUs
      Dim OID
      Set OID = CreateObject("CAPICOM.OID")
      For i = 1 To theEKUs.Count
          If theEKUs(i).Name = CAPICOM_EKU_CLIENT_AUTH Then
              streku = streku & "Client Authentication" & "(" & theEKUs.Item(i).OID & ") "
          ElseIf theEKUs(i).Name = CAPICOM_EKU_CODE_SIGNING Then
              streku = streku & "Code Signing" & "(" & theEKUs.Item(i).OID & ") "
          ElseIf theEKUs(i).Name = CAPICOM_EKU_EMAIL_PROTECTION Then
              streku = streku & "Email" & "(" & theEKUs.Item(i).OID & ") "
          ElseIf theEKUs(i).Name = CAPICOM_EKU_CODE_SIGNING Then
              streku = streku & "Code Signing" & "(" & theEKUs.Item(i).OID & ") "
          ElseIf theEKUs(i).Name = CAPICOM_EKU_SERVER_AUTH Then
              streku = streku & "Server Authentication" & "(" & theEKUs.Item(i).OID & ") "
          ElseIf theEKUs(i).Name = CAPICOM_EKU_SMARTCARD_LOGON Then
              streku = streku & "SmartCard logon" & "(" & theEKUs.Item(i).OID & ") "
          ElseIf theEKUs(i).Name = CAPICOM_EKU_ENCRYPTING_FILE_SYSTEM Then
              streku = streku & "Encrypting File System" & "(" & theEKUs.Item(i).OID & ") "
          Else
              OID.Value = theEKUs.Item(i).OID
              streku = streku & OID.FriendlyName & " (" & OID.Value & ")"
          End If
      Next
      Set OID = Nothing
      WScript.Echo "  Extended Key Usage: Critical = " & cert.ExtendedKeyUsage.IsCritical
      WScript.Echo streku
    End If

    ' Clear out any errors that might have occured while displaying the cert
    Err.Clear

    WScript.Echo
end sub

'******************************************************************************
'
' Function  : IsReDimed
'
' Synopsis  : Check to see if an array has any element.
'
' Parameter : Array - array to check.
'
' Return    : True if contains element, else False.
'
'******************************************************************************

Function IsReDimed (Array)
   ' Always requires this
   On Error Resume Next


   Dim i : i = LBound(Array)
   If Err.Number = 0 Then
      IsReDimed = True
   Else
      Err.Clear
      IsReDimed = False
   End If

End Function ' End IsReDimed


'******************************************************************************
'
' Subroutine: LogError
'
' Synopsis  : Display, log or messagebox an error that just occurred.
'
' Parameter : ErrNum       - The error number.
'             ErrDesc      - The text description of the error.
'             LocationText - Text that describes what the program was attempting to do
'                            when the error occurred.
'             Verbose      - Display errors to the user as well.
'
' Notes :     Uses global WshShell object
'             The Err object also has the following properties but they do not typically 
'             have useful information in vbs:
'               Err.HelpContext Long integer pointing to a context in a help file.
'               Err.HelpFile    String containing the path to the relevant help file.
'               Err.Source      The name of the object causing the error.
'******************************************************************************
Sub LogError (ErrNum, ErrDesc, LocationText, Verbose)
If Not MY_DEBUG Then : On Error Resume Next : End If

' Constants used for logging events to the event log
'Const LogEventSUCCESS        = 0 ' May show up as type information or none
Const LogEventERROR          = 1 
'Const LogEventWARNING        = 2
'Const LogEventINFORMATION    = 4
'Const LogEventAUDIT_SUCCESSS = 8
'Const LogEventAUDIT_FAILURE  = 16

  Dim ErrorString

  ' Create a string with all the information that we will want.
  '   application_name location_in_script (err#, hex_err#) description
  ErrorString = AppName & " " & LocationText & " (" & Cstr(ErrNum) & _
    ", 0x" & hex(ErrNum) & ") " & ErrDesc
  
  ' Sent it to the Application Event Log
  '   On Win 9x the event is added to the WSH.log in the Windows directory.
  WshShell.LogEvent LogEventERROR, ErrorString
  ' If is possible to get an error here, particularly on Win 9x if the %windir%\wsh.log file is locked.
  ' We could check for the error but what would we do with it? 
  ' We could display something on the screen or pop up a message box.

  If Verbose Then
    WScript.Echo "ERROR: " & ErrorString
  End If

End Sub ' End LogError

'******************************************************************************
Sub printUsage()
If Not MY_DEBUG Then : On Error Resume Next : End If
    WScript.Echo "Usage: cscript enroll.vbs [options]"
    WScript.Echo "Display options:"
    WScript.Echo "  /?                   This help screen"
    WScript.Echo "  /h                   This help screen"
    WScript.Echo "  /v                   Verbose mode"
    WScript.Echo "Replacement check options:"
    WScript.Echo "  /certtype <type>     REQUIRED: The Certificate template name (no spaces in the name)"
    WScript.Echo "  /force               Enroll for a cert regardless of if one already exists"
    WScript.Echo "  /eku <oid>       Do not enroll the cert if an existing one is found with this EKU"
    WScript.Echo "  /apppolicy <oid>    Do not enroll the cert if an existing one is found with this Application Policy"
    WScript.Echo "  /certpolicy <oid>    Do not enroll the cert if an existing one is found with this Certificate Policy"
    WScript.Echo "Request options:"
    WScript.Echo "  /ca <servername>     Hostname of CA to enroll against"
    WScript.Echo "  /fn                  Add a Friendly Name to the certificate request"
    WScript.Echo "  /csp                 Cryptographic Service Provider: base, enhanced, strong"
    WScript.Echo "  /keyl                Key length: 512, 1024, 2048, 4096, 8192, 16384"
    WScript.Echo "  /keyusage <type>     Either 1 for AT_KEYEXCHANGE or 2 for AT_SIGNATURE. Default is " & keyUsage
    WScript.Echo "  /signapppol <oid>   Sign the certificate request with a certificate that has this policy"
    WScript.Echo "  /signcertpol <oid>   Sign the certificate request with a certificate that has this policy"
    WScript.Echo "  /strongkey           Enable Strong Private Key Protection"
    WScript.Echo "  /export              Exportable"
    WScript.Echo "  /renew ##            Renew the certificate that matches search criteria (i.e. /apppolicy, /eku, /certpolicy) and the period (default=42)"
    WScript.Echo "  /archive             Archive the private key associated with the request at the CA, requires /export for Windows 2000 clients"
    WScript.Echo ""
    WScript.Echo "Note /force will override /eku, /apppolicy and /certpolicy"
    WScript.Echo ""
    WScript.Echo "Examples:"
    WScript.Echo "Request a certificate template named SigningCert"
    WScript.Echo "cscript enroll.vbs /certtype SigningCert"
    WScript.Echo "                                            "
    WScript.Echo "Request a certificate template named SigningCert that implements strong private key protection"
    WScript.Echo "cscript enroll.vbs /certtype SigningCert /strongkey"
    WScript.Echo ""
    WScript.Echo "Request a certificate template named CustomEFS that enables key archival on Windows XP"
    WScript.Echo "cscript enroll.vbs /certtype CustomEFS /archive"
    WScript.Echo ""
    WScript.Echo "Request a certificate template named CustomEFS that enables key archival on Windows 2000"
    WScript.Echo "cscript enroll.vbs /certtype CustomEFS /archive /export"
    WScript.Echo ""
    WScript.Echo "Request a certificate template named SigningCert only if the user does not have an existing Signing Cert"
    WScript.Echo "The Signing Certificate has the Application policy OIDS: 1.3.6.1.5.5.7.3.2 and 2.16.840.1.999999999.1.1.3"
    WScript.Echo "cscript enroll.vbs /certtype SigningCert /apppolicy 1.3.6.1.5.5.7.3.2 /apppolicy 2.16.840.1.999999999.1.1.3"
    WScript.Echo ""
    WScript.Echo "Renew a certificate based on the SigningCert (THIS WILL USE A NEW KEY PAIR)"
    WScript.Echo "cscript enroll.vbs /certtype SigningCert /apppolicy 1.3.6.1.5.5.7.3.2 /apppolicy 2.16.840.1.999999999.1.1.3 /renew 42"
    WScript.Echo ""

End Sub ' End printUsage


'' SIG '' Begin signature block
'' SIG '' MIIWfwYJKoZIhvcNAQcCoIIWcDCCFmwCAQExCzAJBgUr
'' SIG '' DgMCGgUAMGcGCisGAQQBgjcCAQSgWTBXMDIGCisGAQQB
'' SIG '' gjcCAR4wJAIBAQQQTvApFpkntU2P5azhDxfrqwIBAAIB
'' SIG '' AAIBAAIBAAIBADAhMAkGBSsOAwIaBQAEFCmfANS82RPr
'' SIG '' TO8lG8anpwzqH/PsoIIR5jCCAj0wggGmAhEAzbp/VvDf
'' SIG '' 5LxU/iKss3KqVTANBgkqhkiG9w0BAQIFADBfMQswCQYD
'' SIG '' VQQGEwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4x
'' SIG '' NzA1BgNVBAsTLkNsYXNzIDEgUHVibGljIFByaW1hcnkg
'' SIG '' Q2VydGlmaWNhdGlvbiBBdXRob3JpdHkwHhcNOTYwMTI5
'' SIG '' MDAwMDAwWhcNMjgwODAxMjM1OTU5WjBfMQswCQYDVQQG
'' SIG '' EwJVUzEXMBUGA1UEChMOVmVyaVNpZ24sIEluYy4xNzA1
'' SIG '' BgNVBAsTLkNsYXNzIDEgUHVibGljIFByaW1hcnkgQ2Vy
'' SIG '' dGlmaWNhdGlvbiBBdXRob3JpdHkwgZ8wDQYJKoZIhvcN
'' SIG '' AQEBBQADgY0AMIGJAoGBAOUZv22jVmEtmUhx9mfeuY3r
'' SIG '' t56GgAqRDvo4Ja9GiILlc6igmyRdDR/MZW4MsNBWhBiH
'' SIG '' mgabEKFz37RYOWtuwfYV1aioP6oSBo0xrH+wNNePNGeI
'' SIG '' Cc0UEeJORVZpH3gCgNrcR5EpuzbJY1zF4Ncth3uhtzKw
'' SIG '' ezC6Ki8xqu6jZ9rbAgMBAAEwDQYJKoZIhvcNAQECBQAD
'' SIG '' gYEATD+4i8Zo3+5DMw5d6abLB4RNejP/khv0Nq3YlSI2
'' SIG '' aBFsfELM85wuxAc/FLAPT/+Qknb54rxK6Y/NoIAK98Up
'' SIG '' 8YIiXbix3YEjo3slFUYweRb46gVLlH8dwhzI47f0EEA8
'' SIG '' E8NfH1PoSOSGtHuhNbB7Jbq4046rPzidADQAmPPRcZQw
'' SIG '' ggNmMIICz6ADAgECAhANi0/uqtIYW/R1ap0p4X/7MA0G
'' SIG '' CSqGSIb3DQEBAgUAMF8xCzAJBgNVBAYTAlVTMRcwFQYD
'' SIG '' VQQKEw5WZXJpU2lnbiwgSW5jLjE3MDUGA1UECxMuQ2xh
'' SIG '' c3MgMSBQdWJsaWMgUHJpbWFyeSBDZXJ0aWZpY2F0aW9u
'' SIG '' IEF1dGhvcml0eTAeFw05ODA1MTIwMDAwMDBaFw0wODA1
'' SIG '' MTIyMzU5NTlaMIHMMRcwFQYDVQQKEw5WZXJpU2lnbiwg
'' SIG '' SW5jLjEfMB0GA1UECxMWVmVyaVNpZ24gVHJ1c3QgTmV0
'' SIG '' d29yazFGMEQGA1UECxM9d3d3LnZlcmlzaWduLmNvbS9y
'' SIG '' ZXBvc2l0b3J5L1JQQSBJbmNvcnAuIEJ5IFJlZi4sTElB
'' SIG '' Qi5MVEQoYyk5ODFIMEYGA1UEAxM/VmVyaVNpZ24gQ2xh
'' SIG '' c3MgMSBDQSBJbmRpdmlkdWFsIFN1YnNjcmliZXItUGVy
'' SIG '' c29uYSBOb3QgVmFsaWRhdGVkMIGfMA0GCSqGSIb3DQEB
'' SIG '' AQUAA4GNADCBiQKBgQC7WkSKBBa7Vf0DeootlE8VeDa4
'' SIG '' DUqyb5xUv7zodyqdufBou5XZMUFweoFLuUgTVi3HCOGE
'' SIG '' QqvAopKrRFyqQvCCDgLpL/vCO7u+yScKXbawNkIztW5U
'' SIG '' iE+HSr8Z2vkV6A+HthzjzMaajn9qJJLj/OBluqexfu/J
'' SIG '' 2zdqyErICQbkmQIDAQABo4G0MIGxMBEGCWCGSAGG+EIB
'' SIG '' AQQEAwIBBjA1BgNVHR8ELjAsMCqgKKAmhiRodHRwOi8v
'' SIG '' Y3JsLnZlcmlzaWduLmNvbS9wY2ExLjEuMS5jcmwwRwYD
'' SIG '' VR0gBEAwPjA8BgtghkgBhvhFAQcBATAtMCsGCCsGAQUF
'' SIG '' BwIBFh93d3cudmVyaXNpZ24uY29tL3JlcG9zaXRvcnkv
'' SIG '' UlBBMA8GA1UdEwQIMAYBAf8CAQAwCwYDVR0PBAQDAgEG
'' SIG '' MA0GCSqGSIb3DQEBAgUAA4GBAEJ8Dt+MeUysvwjsTVUv
'' SIG '' UImgxV5OLl6VMpt5rWURCxxKUsTVqDEhjt4Qm2wIxQfm
'' SIG '' A7nnyDR4CQnyvAZC+FqMg9GK3qoi9dnjIdLPZYwGM7DN
'' SIG '' ILIzzQq9PuGdwTWpZLCnpSRb6fFo6xPEfDf0lGQNmsW9
'' SIG '' MxfvgzOgPuWqPq7Ycx+tMIIDxDCCAy2gAwIBAgIQR78Z
'' SIG '' ld+NUkZD99ttSA0xpDANBgkqhkiG9w0BAQUFADCBizEL
'' SIG '' MAkGA1UEBhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2Fw
'' SIG '' ZTEUMBIGA1UEBxMLRHVyYmFudmlsbGUxDzANBgNVBAoT
'' SIG '' BlRoYXd0ZTEdMBsGA1UECxMUVGhhd3RlIENlcnRpZmlj
'' SIG '' YXRpb24xHzAdBgNVBAMTFlRoYXd0ZSBUaW1lc3RhbXBp
'' SIG '' bmcgQ0EwHhcNMDMxMjA0MDAwMDAwWhcNMTMxMjAzMjM1
'' SIG '' OTU5WjBTMQswCQYDVQQGEwJVUzEXMBUGA1UEChMOVmVy
'' SIG '' aVNpZ24sIEluYy4xKzApBgNVBAMTIlZlcmlTaWduIFRp
'' SIG '' bWUgU3RhbXBpbmcgU2VydmljZXMgQ0EwggEiMA0GCSqG
'' SIG '' SIb3DQEBAQUAA4IBDwAwggEKAoIBAQCpyrKkzM0grwp9
'' SIG '' iayHdfC0TvHfwQ+/Z2G9o2Qc2rv5yjOrhDCJWH6M22vd
'' SIG '' Np4Pv9HsePJ3pn5vPL+Trw26aPRslMq9Ui2rSD31ttVd
'' SIG '' XxsCn/ovax6k96OaphrIAuF/TFLjDmDsQBx+uQ3eP8e0
'' SIG '' 34e9X3pqMS4DmYETqEcgzjFzDVctzXg0M5USmRK53mgv
'' SIG '' qubjwoqMKsOLIYdmvYNYV291vzyqJoddyhAVPJ+E6lTB
'' SIG '' Cm7E/sVK3bkHEZcifNs+J9EeeOyfMcnx5iIZ28SzR0Oa
'' SIG '' Gl+gHpDkXvXufPF9q2IBj/VNC97QIlaolc2uiHau7roN
'' SIG '' 8+RN2aD7aKCuFDuzh8G7AgMBAAGjgdswgdgwNAYIKwYB
'' SIG '' BQUHAQEEKDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2Nz
'' SIG '' cC52ZXJpc2lnbi5jb20wEgYDVR0TAQH/BAgwBgEB/wIB
'' SIG '' ADBBBgNVHR8EOjA4MDagNKAyhjBodHRwOi8vY3JsLnZl
'' SIG '' cmlzaWduLmNvbS9UaGF3dGVUaW1lc3RhbXBpbmdDQS5j
'' SIG '' cmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/
'' SIG '' BAQDAgEGMCQGA1UdEQQdMBukGTAXMRUwEwYDVQQDEwxU
'' SIG '' U0EyMDQ4LTEtNTMwDQYJKoZIhvcNAQEFBQADgYEASmv5
'' SIG '' 6ljCRBwxiXmZK5a/gqwB1hxMzbCKWG7fCCmjXsjKkxPn
'' SIG '' BFIN70cnLwA4sOTJk06a1CJiFfc/NyFPcDGA8Ys4h7Po
'' SIG '' 6JcA/s9Vlk4k0qknTnqut2FB8yrO58nZXt27K4U+tZ21
'' SIG '' 2eFX/760xX71zwye8Jf+K9M7UhsbOCf3P0owggP/MIIC
'' SIG '' 56ADAgECAhAN6Svw1NgpiBgyBQlemnaIMA0GCSqGSIb3
'' SIG '' DQEBBQUAMFMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5W
'' SIG '' ZXJpU2lnbiwgSW5jLjErMCkGA1UEAxMiVmVyaVNpZ24g
'' SIG '' VGltZSBTdGFtcGluZyBTZXJ2aWNlcyBDQTAeFw0wMzEy
'' SIG '' MDQwMDAwMDBaFw0wODEyMDMyMzU5NTlaMFcxCzAJBgNV
'' SIG '' BAYTAlVTMRcwFQYDVQQKEw5WZXJpU2lnbiwgSW5jLjEv
'' SIG '' MC0GA1UEAxMmVmVyaVNpZ24gVGltZSBTdGFtcGluZyBT
'' SIG '' ZXJ2aWNlcyBTaWduZXIwggEiMA0GCSqGSIb3DQEBAQUA
'' SIG '' A4IBDwAwggEKAoIBAQCyUChI3dNoeoQYRGZ1XX7EuJ9j
'' SIG '' Jv89Q5x8ETgQJVVz2XUnaf1OuSBc0wr5oBsq7VVWIWHY
'' SIG '' HtvkvDNrx+/dozdljhuTDLZTHlx8ZjVfBYpF/nZO31OA
'' SIG '' ooEgna6IXKII9+Uw+e4iN0xCCs7fxh/E1lXpgT+1UqMs
'' SIG '' qgF68qKqjTX+n+ZdagWfPWvjv5bA/sxg+UDnB6BE64FR
'' SIG '' bqUq8raKECjtj9wGoIZQmntKCA0wHcoQnmv36ViuBKlA
'' SIG '' mbIo6I8WrDzjU29L0zWdtW9kHbOWLLs953nrbXr5FuYm
'' SIG '' ra/vmVO3QCyVuHmq/tRSqyl0fkLsOR6iahbmWbskaNgA
'' SIG '' gEMQh4BrAgMBAAGjgcowgccwNAYIKwYBBQUHAQEEKDAm
'' SIG '' MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC52ZXJpc2ln
'' SIG '' bi5jb20wDAYDVR0TAQH/BAIwADAzBgNVHR8ELDAqMCig
'' SIG '' JqAkhiJodHRwOi8vY3JsLnZlcmlzaWduLmNvbS90c3Mt
'' SIG '' Y2EuY3JsMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMIMA4G
'' SIG '' A1UdDwEB/wQEAwIGwDAkBgNVHREEHTAbpBkwFzEVMBMG
'' SIG '' A1UEAxMMVFNBMjA0OC0xLTU0MA0GCSqGSIb3DQEBBQUA
'' SIG '' A4IBAQCHeHDaTlIBIFvgecmCMMT9uRmWvZEAw73Nzcb0
'' SIG '' Dtj/+U3AM2IwEcX1dBvUkt5fnCATsXxFvlDNg+eAF4On
'' SIG '' J5NnE0b7yriYQQPMm1FbBYt/qG/zG1AbJC7yaY1sIve7
'' SIG '' yhaV7Qx0wGh32euZYofBc5D4iXR6I6ujmHuXsfePKXFN
'' SIG '' LnUbSEHa8LUNIFTWd6CXgmNp/QnPivB1uwmb2fkRVSaa
'' SIG '' YTK+egKwe4a+osOLIix40TV2vJJzXPm55kwVCiPM5NLU
'' SIG '' NC5JQBU8D2B6JMalZu+Wz3DrPuf0DX7c0XyjdnFpwZxP
'' SIG '' RzA1IbGirxpiPCvZjqoqB3vYGLNce+KdpW/+PImtMIIE
'' SIG '' bDCCA9WgAwIBAgIQSrYqkfvsQ7Ee57d49kEl7TANBgkq
'' SIG '' hkiG9w0BAQQFADCBzDEXMBUGA1UEChMOVmVyaVNpZ24s
'' SIG '' IEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5l
'' SIG '' dHdvcmsxRjBEBgNVBAsTPXd3dy52ZXJpc2lnbi5jb20v
'' SIG '' cmVwb3NpdG9yeS9SUEEgSW5jb3JwLiBCeSBSZWYuLExJ
'' SIG '' QUIuTFREKGMpOTgxSDBGBgNVBAMTP1ZlcmlTaWduIENs
'' SIG '' YXNzIDEgQ0EgSW5kaXZpZHVhbCBTdWJzY3JpYmVyLVBl
'' SIG '' cnNvbmEgTm90IFZhbGlkYXRlZDAeFw0wNDA5MDQwMDAw
'' SIG '' MDBaFw0wNTA5MDQyMzU5NTlaMIIBEDEXMBUGA1UEChMO
'' SIG '' VmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWdu
'' SIG '' IFRydXN0IE5ldHdvcmsxRjBEBgNVBAsTPXd3dy52ZXJp
'' SIG '' c2lnbi5jb20vcmVwb3NpdG9yeS9SUEEgSW5jb3JwLiBi
'' SIG '' eSBSZWYuLExJQUIuTFREKGMpOTgxHjAcBgNVBAsTFVBl
'' SIG '' cnNvbmEgTm90IFZhbGlkYXRlZDE0MDIGA1UECxMrRGln
'' SIG '' aXRhbCBJRCBDbGFzcyAxIC0gTWljcm9zb2Z0IEZ1bGwg
'' SIG '' U2VydmljZTEUMBIGA1UEAxQLQnJpYW4gS29tYXIxIDAe
'' SIG '' BgkqhkiG9w0BCQEWEWJrb21hckBpZGVudGl0LmNhMIGf
'' SIG '' MA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDsPsVNgIuL
'' SIG '' P3D41QWLNjgsh58VfQOuFi2shnDMN717k0lYkcwruTvK
'' SIG '' O+bVZfaH5UOCjAFdtvzSnIGUebq0cVwjt/1T3oVI9dsL
'' SIG '' e0BpVxasrGLlh7PNwugiu3pZnEXV25ziLWFVOd/ACddz
'' SIG '' CKZJx0wVYU5fB1+ulak0Rl+car0j0wIDAQABo4IBBjCC
'' SIG '' AQIwCQYDVR0TBAIwADCBrAYDVR0gBIGkMIGhMIGeBgtg
'' SIG '' hkgBhvhFAQcBATCBjjAoBggrBgEFBQcCARYcaHR0cHM6
'' SIG '' Ly93d3cudmVyaXNpZ24uY29tL0NQUzBiBggrBgEFBQcC
'' SIG '' AjBWMBUWDlZlcmlTaWduLCBJbmMuMAMCAQEaPVZlcmlT
'' SIG '' aWduJ3MgQ1BTIGluY29ycC4gYnkgcmVmZXJlbmNlIGxp
'' SIG '' YWIuIGx0ZC4gKGMpOTcgVmVyaVNpZ24wEQYJYIZIAYb4
'' SIG '' QgEBBAQDAgeAMDMGA1UdHwQsMCowKKAmoCSGImh0dHA6
'' SIG '' Ly9jcmwudmVyaXNpZ24uY29tL2NsYXNzMS5jcmwwDQYJ
'' SIG '' KoZIhvcNAQEEBQADgYEAYRdCTvyefyOy9aIrd5OUhg7r
'' SIG '' X3uNRgolVJ0CloQQ8n5cg7Ti1Fgn7395+xO8Tn8BNRyP
'' SIG '' 0yYvlaxKt6UOFt70q/sfcPvK+P26leHvb5JeSx+NPJcK
'' SIG '' 1VJAILwa6aRbEmUyDQml45QNPH6dB7rW1J0JI273CXyj
'' SIG '' OiH31MuDBqhFkhUxggQFMIIEAQIBATCB4TCBzDEXMBUG
'' SIG '' A1UEChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZl
'' SIG '' cmlTaWduIFRydXN0IE5ldHdvcmsxRjBEBgNVBAsTPXd3
'' SIG '' dy52ZXJpc2lnbi5jb20vcmVwb3NpdG9yeS9SUEEgSW5j
'' SIG '' b3JwLiBCeSBSZWYuLExJQUIuTFREKGMpOTgxSDBGBgNV
'' SIG '' BAMTP1ZlcmlTaWduIENsYXNzIDEgQ0EgSW5kaXZpZHVh
'' SIG '' bCBTdWJzY3JpYmVyLVBlcnNvbmEgTm90IFZhbGlkYXRl
'' SIG '' ZAIQSrYqkfvsQ7Ee57d49kEl7TAJBgUrDgMCGgUAoHgw
'' SIG '' GQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwIwYJKoZI
'' SIG '' hvcNAQkEMRYEFCnprdHOkIwU8LGJZyiAMGsAt3M2MDYG
'' SIG '' CisGAQQBgjcCAQwxKDAmoCSAIgBFAG4AcgBvAGwAbABt
'' SIG '' AGUAbgB0ACAAUwBjAHIAaQBwAHQwDQYJKoZIhvcNAQEB
'' SIG '' BQAEgYBKVcrCRypjb+EgMnfr++fbSvY4l6UQ6XTIiI8G
'' SIG '' WV/sVs27QKEJAT0ZCZ1oGKkePjGZLbibfhnU384ly/gw
'' SIG '' qgpTUztdY61CiMGt41A5N8VsqG8f2Ib1mGkIeFGOtCvA
'' SIG '' lhHEi2Sss2dPrltUD4LkDuGxTYUMbYUBJPupJ9Tb7MT+
'' SIG '' FaGCAf8wggH7BgkqhkiG9w0BCQYxggHsMIIB6AIBATBn
'' SIG '' MFMxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5WZXJpU2ln
'' SIG '' biwgSW5jLjErMCkGA1UEAxMiVmVyaVNpZ24gVGltZSBT
'' SIG '' dGFtcGluZyBTZXJ2aWNlcyBDQQIQDekr8NTYKYgYMgUJ
'' SIG '' Xpp2iDAMBggqhkiG9w0CBQUAoFkwGAYJKoZIhvcNAQkD
'' SIG '' MQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMDUw
'' SIG '' MTA1MjAwOTM5WjAfBgkqhkiG9w0BCQQxEgQQ5i12M/4f
'' SIG '' o2rxx93onPwW5jANBgkqhkiG9w0BAQEFAASCAQCbhiPc
'' SIG '' nS9O3B1OX3OWHTmsffM/buXzT3P1UZM1YRrlS56osN2G
'' SIG '' eIPlPkuVK1NOSC6MBpWgs+r7DU7gI1iurK6lqD3xRFzE
'' SIG '' feuxtlhbIxxi4iQy7pfGVWgs8QVAMHSsvcseSWMBVEC7
'' SIG '' UbTkmG0Ud5cIABxooUFIEjKV77ndJvieA2ATsXL7eJFa
'' SIG '' wBREpnvgcuzainUF+I4ghHEUifoT3VKS3oTYU++ysLuS
'' SIG '' yq2TyaKomd1yGK+Y1xuN1l4btWw57lztdBGkN/VNktlm
'' SIG '' pqT5gwVulg4Etn1EotAdGx37NrtKj6ShBYLlpTZnDa86
'' SIG '' aXRhEBonEF23AlV6FT2EFMYi3YhM
'' SIG '' End signature block
