' Certificate Store CAPICOM script

' Requires the installation of CAPICOM.DLL on the client computer
'
' This script is provided "As is", no technical assistance will be offered
' 
' Microsoft(C) Corporation 2003



Option Explicit
On Error Resume Next

    ' This application name should include a version number but should remain short.
    '   It is used when logging errors and such
    Const AppName = "ctool.vbs v01.05a03"

    ' CAPICOM Constants
    ' Certificate store names
    Const CAPICOM_MY_STORE    = "My"
    Const CAPICOM_CA_STORE    = "CA"
    Const CAPICOM_ROOT_STORE  = "Root"
    Const CAPICOM_OTHER_STORE = "AddressBook" 
    ' Store Location Names (CAPICOM_STORE_LOCATION enumeration)
    Const CAPICOM_MEMORY_STORE                                     = 0
    Const CAPICOM_LOCAL_MACHINE_STORE                              = 1
    Const CAPICOM_CURRENT_USER_STORE                               = 2
    Const CAPICOM_ACTIVE_DIRECTORY_USER_STORE                      = 3
    Const CAPICOM_SMART_CARD_USER_STORE                            = 4
    Const MAX_ALLOWED = 2
    Const CAPICOM_STORE_OPEN_READ_ONLY = 0
    Const CAPICOM_CHECK_ONLINE_REVOCATION_STATUS = 8    
    Const CAPICOM_TRUST_IS_REVOKED = 4
    Const CAPICOM_EKU_CLIENT_AUTH = 2
    Const CAPICOM_EKU_CODE_SIGNING = 3
    Const CAPICOM_EKU_EMAIL_PROTECTION = 4
    Const CAPICOM_EKU_SERVER_AUTH = 1
    Const CAPICOM_EKU_OTHER = 0
    ' Find
    Const CAPICOM_CERTIFICATE_FIND_SHA1_HASH                       = 0
    Const CAPICOM_CERTIFICATE_FIND_SUBJECT_NAME                    = 1
    Const CAPICOM_CERTIFICATE_FIND_ISSUER_NAME                     = 2
    Const CAPICOM_CERTIFICATE_FIND_ROOT_NAME                       = 3
    Const CAPICOM_CERTIFICATE_FIND_TEMPLATE_NAME                   = 4
    Const CAPICOM_CERTIFICATE_FIND_EXTENSION                       = 5
    Const CAPICOM_CERTIFICATE_FIND_EXTENDED_PROPERTY               = 6
    Const CAPICOM_CERTIFICATE_FIND_APPLICATION_POLICY              = 7
    Const CAPICOM_CERTIFICATE_FIND_CERTIFICATE_POLICY              = 8
    Const CAPICOM_CERTIFICATE_FIND_TIME_VALID                      = 9
    Const CAPICOM_CERTIFICATE_FIND_TIME_NOT_YET_VALID              = 10
    Const CAPICOM_CERTIFICATE_FIND_TIME_EXPIRED                    = 11
    Const CAPICOM_CERTIFICATE_FIND_KEY_USAGE                       = 12

    ' Constants used for logging events to the event log
'    Const LogEventSUCCESS        = 0 ' May show up as type information or none
    Const LogEventERROR          = 1 
'    Const LogEventWARNING        = 2
'    Const LogEventINFORMATION    = 4
'    Const LogEventAUDIT_SUCCESSS = 8
'    Const LogEventAUDIT_FAILURE  = 16

    Dim certs 
    Dim cert
    Dim newCert
    Dim newCertThumbprint
    Dim i, j
    Dim ch  
    Dim st 
    Dim coll 
    Dim cnt
    Dim args
    Dim status
    Dim action
    Dim newcertcnt 
    Dim bFoundCert
    Dim WshShell
    Dim certCount
    Dim delcount
    Dim filteredCerts

    Set WshShell = WScript.CreateObject("WScript.Shell")

    Set st = CreateObject("CAPICOM.Store")
    If Err.Number <> 0 Then
      WScript.Echo "CAPICOM is not installed. Exiting."
      WScript.Echo "run: regsvr32 capicom.dll"
      LogError Err.Number, Err.Description, "Creating CAPICOM.Store Object", bVerbose
      Err.Clear
      WScript.Quit(1)
    End If
    Set ch = CreateObject("CAPICOM.Chain")
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Creating CAPICOM.Chain Object", bVerbose
      Err.Clear
    End If

    ' Declare and Initialize the command line arguement variables
    ' Delaration             : Initialization
    Dim ValidToDate
    Dim bRemoveExpiredCerts  : bRemoveExpiredCerts   = False
    Dim CertThumbprint       : CertThumbprint        = NULL
    Dim openmode             : openmode              = MAX_ALLOWED
    Dim Subjects()
    Dim filename             : filename              = ""
    Dim bRemoveRevokedCerts  : bRemoveRevokedCerts   = False
    Dim storename            : storename             = CAPICOM_MY_STORE
    Dim storeLocation        : storeLocation         = CAPICOM_CURRENT_USER_STORE
    Dim bVerbose             : bVerbose              = False
    Dim impotent             : impotent              = true
    Dim bNoReplace           : bNoReplace            = False
    Dim AppPolicies()
    Dim CertPolicies()

    delcount = 0

    ' If we don't get any command line args then print the help
    Set args = WScript.Arguments
    if args.count = 0 then
        call printUsage
        WScript.Quit(0)
    end if

    ' Read the command line arguements
    For i = 0 To args.Count - 1
        if strcomp(args(i), "/validto") = 0 then
            ValidToDate = CDate(args(i+1))
            bRemoveExpiredCerts = True
        elseif strcomp(args(i), "/del") = 0 then
            CertThumbprint = args(i+1)
            action = "del"
        elseif strcomp(args(i), "/show") = 0 then
            CertThumbprint = args(i+1)
            action = "show"
        elseif strcomp(args(i), "/list") = 0 then
            action = "list"
            openmode = CAPICOM_STORE_OPEN_READ_ONLY
        elseif strcomp(args(i), "/sha1") = 0 then
            CertThumbprint = args(i+1)
            openmode = CAPICOM_STORE_OPEN_READ_ONLY
        elseif strcomp(args(i), "/subj") = 0 then
            If IsReDimed(Subjects) Then
               ReDim Preserve Subjects(UBound(Subjects) + 1)
            Else
               ReDim Subjects(0)
            End If
            Subjects(UBound(Subjects)) = args(i+1)
            openmode = CAPICOM_STORE_OPEN_READ_ONLY
        elseif strcomp(args(i), "/file") = 0 then
            filename = args(i+1)
        elseif strcomp(args(i), "/add") = 0 then
            filename = args(i+1)
            action = "add"
        elseif strcomp(args(i), "/delrevoked") = 0 then
            bRemoveRevokedCerts = True    
        elseif strcomp(args(i), "/store" ) = 0 then
            storename = args(i+1)
        elseif strcomp(args(i), "/storeloc" ) = 0 then
            if strcomp(args(i+1),"lm") = 0 then
                storeLocation = CAPICOM_LOCAL_MACHINE_STORE
            elseif strcomp(args(i+1), "cu") = 0 then
                storeLocation = CAPICOM_CURRENT_USER_STORE
            end if
        elseif strcomp(args(i), "/verbose" ) = 0 then
            bVerbose = True
        elseif strcomp(args(i), "/remove" ) = 0 then
            impotent = false
        elseif strcomp(args(i), "/noreplace" ) = 0 then
            bNoReplace = True
        elseif strcomp(args(i), "/app_policy") = 0 then
           If IsReDimed(AppPolicies) Then
             ' Make the array a little bigger to fit the new value.
             ReDim Preserve AppPolicies(UBound(AppPolicies)+1)
           Else
             ' First time so don't preserve.
             ReDim AppPolicies(0)
           End If
           AppPolicies(UBound(AppPolicies)) = args(i+1)
        elseif strcomp(args(i), "/certpolicy") = 0 then
           If IsReDimed(CertPolicies) Then
             ' Make the array a little bigger to fit the new value.
             ReDim Preserve CertPolicies(UBound(CertPolicies)+1)
           Else
             ' First time so don't preserve.
             ReDim CertPolicies(0)
           End If
           CertPolicies(UBound(CertPolicies)) = args(i+1)
        end if
    Next 

    st.Open storeLocation, storename, openmode
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Opening Cert store " & _
        storeLocation & "," & storename & "," & openmode, bVerbose
      Err.Clear
    End If

    If action = "del"  Then 
        Set certs = st.Certificates
        For i = 1 To certs.Count
            Set cert = certs(i)
            if bVerbose then
                WScript.Echo "---------------- " & i &  " ----------------"
                PrintCert cert, bVerbose
            End if
            If StrComp(left(cert.Thumbprint,40), CertThumbprint) = 0 Then
                if not impotent then
                    st.Remove cert
                end if
                if Err.Number <> 0 Then
                    WScript.Echo "Error processing certificate when removing by Thumbprint: " 
                    LogError Err.Number, Err.Description, "Error processing certificate when removing by Thumbprint", bVerbose
                    PrintCert cert, bVerbose
                    WScript.Echo Err.Description
                    Err.Clear
                else
                    WScript.Echo "Removed the following certificate:" 
                    PrintCert cert, bVerbose
                    if not impotent then
                        delcount = delcount+1
                    end if
                end if
            End If
 
            If bRemoveExpiredCerts Then
                If cert.ValidToDate < ValidToDate Then
                    if not impotent then
                        st.Remove cert
                    end if
                    if Err.Number <> 0 Then
                        WScript.Echo "Error processing certificate when removing by Expiration Date: " 
                        LogError Err.Number, Err.Description, "Error processing certificate when removing by Expiration Date", bVerbose
                        PrintCert cert, bVerbose
                        WScript.Echo Err.Description
                        Err.Clear
                    else
                        WScript.Echo "Removed the following certificate:" 
                        PrintCert cert, bVerbose
                        if not impotent then
                            delcount = delcount+1
                        end if
                    end if
                End If
            End If
            
            If bRemoveRevokedCerts Then
                ch.build cert
                If Err.Number <> 0 Then
                  LogError Err.Number, Err.Description, "Building Cert during revok", bVerbose
                  Err.Clear
                End If
                If ch.status And CAPICOM_TRUST_IS_REVOKED  Then
                    if not impotent then
                        st.Remove cert
                    end if
                    if Err.Number <> 0 Then
                        WScript.Echo "Error processing certificate when removing by revocation error:"
                        LogError Err.Number, Err.Description, "Error processing certificate when removing by revocation error", bVerbose
                        PrintCert cert, bVerbose
                        WScript.Echo Err.Description
                        Err.Clear
                    else
                        WScript.Echo "" 
                        WScript.Echo "Removed the following certificate:" 
                        PrintCert cert, bVerbose
                        if not impotent then
                            delcount = delcount+1
                        end if
                    end if
                End If
            End If
        Next 
        WScript.Echo delcount & " certificate(s) were removed"

    ElseIf action = "add" then 
        newcertcnt = GetCertCount (filename)
        for i = 1 to newcertcnt
            Set newCert = CreateObject("CAPICOM.Certificate")
            If Err.Number <> 0 Then
              LogError Err.Number, Err.Description, "Creating CAPICOM.Certificate object", bVerbose
              Err.Clear
            End If
            'WScript.Echo GetCertFromFile (filename , i)
            newCert.Import GetCertFromFile (filename , i)
            If Err.Number <> 0 Then
              LogError Err.Number, Err.Description, "Importing Certificate", bVerbose
              Err.Clear             
            End If
            if bVerbose Then
                PrintCert newCert, bVerbose
            End If
            if bNoReplace Then
              ' Do not add the certificate if we find one with the same thumbprint
              Set certs = st.Certificates
              bFoundCert = False
              newCertThumbprint = left(newCert.Thumbprint,40)
              For j = 1 To certs.Count
                  Set cert = certs(i)
                  If StrComp(left(cert.Thumbprint,40), newCertThumbprint) = 0 Then
                      bFoundCert = True
                      if bVerbose Then
                          WScript.Echo "Found Certificate with same thumbprint, new certificate will not be added."
                      End If
                  End If
              Next
              if bFoundCert <> True Then
                  st.Add newCert
                  If Err.Number <> 0 Then
                      LogError Err.Number, Err.Description, "Adding Certificate", bVerbose
                      Err.Clear
                  End If                
              End If 
            Else
              st.Add newCert
              If Err.Number <> 0 Then
                LogError Err.Number, Err.Description, "Adding Certificate", bVerbose
                Err.Clear
              End If
            End If

        Next 
        WScript.Echo  newcertcnt & " certificates added to the " & storename & " store."

    ElseIf action = "list" Then
        Set certs = st.Certificates
        If filename <> "" Then
            Set cert = CreateObject("CAPICOM.Certificate")
            If Err.Number <> 0 Then
              LogError Err.Number, Err.Description, "Creating CAPICOM.Certificate object", bVerbose
              Err.Clear
            End If
            cert.Import GetCertFromFile(filename,1)
            If Err.Number <> 0 Then
              LogError Err.Number, Err.Description, "Importing Certificate", bVerbose
              Err.Clear             
            End If
            WScript.Echo "----------------------------------"
            PrintCert cert, bVerbose
        Else
            ' Filter down to only the certs that include the attributes specified
            Set filteredCerts = FilterCertificates(st.Certificates)
            cnt = 0
            For Each cert In filteredCerts
                cnt = cnt + 1
                WScript.Echo "---------------- " & cnt &  " ----------------"
                PrintCert cert, bVerbose
            Next
        End If

    ElseIf action = "show" Then
        Set certs = st.Certificates
        For i = 1 To certs.Count
            Set cert = certs(i)
            If StrComp(left(cert.Thumbprint,40), CertThumbprint) = 0 Then
                cert.Display
                If Err.Number <> 0 Then
                  LogError Err.Number, Err.Description, "Displaying Certificate", bVerbose
                  Err.Clear             
                End If
            End If
        Next
    Else 
        ' We didn't recognize the action so print the help
        printUsage()
    End if

    ' Cleanup
    Set st = Nothing
    Set ch = Nothing
    Set args = Nothing
    Set certs = Nothing
    Set cert = Nothing
    Set newCert = Nothing
    Set WshShell = Nothing

    WScript.Quit(0)

'******************************************************************************
Sub PrintCert (cert, Verbose)
    Dim theEKUs
    Dim i
    Dim streku
    if Verbose Then
        WScript.Echo "  Logical Store: " & storename
        if storeLocation = CAPICOM_CURRENT_USER_STORE then
            WScript.Echo "  Store Location: User Store"
        else
            WScript.Echo "  Store Location: Machine Store"
        end if
        WScript.Echo "  Subject Name: " & cert.SubjectName
        WScript.Echo "  Issuer Name: " & cert.IssuerName
        WScript.Echo "  SHA-1 Thumbprint: " & cert.Thumbprint
        WScript.Echo "  Serial Number: " & cert.SerialNumber
        WScript.Echo "  Version: " & cert.Version
        WScript.Echo "  Valid From: " & cert.ValidFromDate
        WScript.Echo "  Valid To: " & cert.ValidToDate
    Else
        WScript.Echo "  Subject Name: " & cert.SubjectName
        WScript.Echo "  SHA-1 Thumbprint: " & cert.Thumbprint
        WScript.Echo "  Valid To: " & cert.ValidToDate
    End If
    Set theEKUs = cert.ExtendedKeyUsage.EKUs
    For i = 1 To theEKUs.Count
        If theEKUs(i).Name = CAPICOM_EKU_CLIENT_AUTH Then
            streku = streku & "Client Authentication" & "(" & theEKUs.Item(i).OID & ") "
        ElseIf theEKUs(i).Name = CAPICOM_EKU_CODE_SIGNING Then
            streku = streku & "Code Signing" & "(" & theEKUs.Item(i).OID & ") "
        ElseIf theEKUs(i).Name = CAPICOM_EKU_EMAIL_PROTECTION Then
            streku = streku & "Email" & "(" & theEKUs.Item(i).OID & ") "
        ElseIf theEKUs(i).Name = CAPICOM_EKU_CODE_SIGNING Then
            streku = streku & "Code Signing" & "(" & theEKUs.Item(i).OID & ") "
        ElseIf theEKUs(i).Name = CAPICOM_EKU_SERVER_AUTH Then
            streku = streku & "Server Authentication" & "(" & theEKUs.Item(i).OID & ") "
        Else
            streku = streku & "Unknown" & "(" & theEKUs.Item(i).OID & ") "
        End If
    Next
    WScript.Echo "  Extended Key Usage: " & streku
    WScript.Echo
end sub

'******************************************************************************
'
' Function  : FilterCertificates
'
' Synopsis  : Filter the set of certificates based on the filtering options.
'
' Parameter : Certificates - The certificates collection to be filtered.
'
' Return    : The filtered certificates collection.
'
' Notes     : Uses globals CertThumbprint, Subjects, AppPolicies and CertPolicies
'
'******************************************************************************

Function FilterCertificates (Certificates)
   Dim iIndex
   Dim Filtered
   Set Filtered = Certificates

   ' If only makes sense to look at one thumbprint
   If Filtered.Count > 0 AND Not IsNull(CertThumbprint) Then
       Set Filtered = Filtered.Find(CAPICOM_CERTIFICATE_FIND_SHA1_HASH, CertThumbprint)
       If Err.Number <> 0 Then
         LogError Err.Number, Err.Description, "Filtering Certificates on Thumbprint", bVerbose
         Err.Clear
       End If
   End If

   If Filtered.Count > 0 AND IsReDimed(Subjects) Then
         If Err.Number <> 0 Then
           LogError Err.Number, Err.Description, "Filtering Certificates on Subject name", bVerbose
           Err.Clear
         End If
      For iIndex = LBound(Subjects) To UBound(Subjects) 
         Set Filtered = Filtered.Find(CAPICOM_CERTIFICATE_FIND_SUBJECT_NAME, Subjects(iIndex))
         If Err.Number <> 0 Then
           LogError Err.Number, Err.Description, "Filtering Certificates on Subject name", bVerbose
           Err.Clear
         End If
      Next
   End If
   
   If Filtered.Count > 0 AND IsReDimed(AppPolicies) Then
      For iIndex = LBound(AppPolicies) To UBound(AppPolicies) 
         Set Filtered = Filtered.Find(CAPICOM_CERTIFICATE_FIND_APPLICATION_POLICY, AppPolicies(iIndex))
         If Err.Number <> 0 Then
           LogError Err.Number, Err.Description, "Filtering Certificates on Policy", bVerbose
           Err.Clear
         End If
      Next
   End If

   If Filtered.Count > 0 AND IsReDimed(CertPolicies) Then
      For iIndex = LBound(CertPolicies) To UBound(CertPolicies) 
         Set Filtered = Filtered.Find(CAPICOM_CERTIFICATE_FIND_CERTIFICATE_POLICY, CertPolicies(iIndex))
         If Err.Number <> 0 Then
           LogError Err.Number, Err.Description, "Filtering Certificates on Certificate Policy", bVerbose
           Err.Clear
         End If
      Next
   End If
   
   Set FilterCertificates = Filtered

End Function ' End FilterCertificates

'******************************************************************************
Function GetCertCount ( filename )
    Dim cnt
    Dim txtstr
    Dim s
    Dim fso
    Dim fContinue

    fContinue = True
    cnt = 0
    set fso = CreateObject("Scripting.FileSystemObject")
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Creating Scripting.FileSystemObject", bVerbose
      Err.Clear             
    End If
    Set txtstr = fso.OpenTextFile(filename)
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "OpenTextFile(" & filename & ")", bVerbose
      Err.Clear             
    End If    
    Do
        s = txtstr.ReadLine()
        'WScript.Echo s
        if txtstr.AtEndOfStream Then
            fContinue = false
        end if
        if strcomp(s, "-----BEGIN CERTIFICATE-----")=0 then
            cnt = cnt + 1
        end if
    Loop While fContinue
    GetCertCount = cnt
End Function

'******************************************************************************
Function GetCertFromFile (filename, index)
    Dim cnt
    Dim txtstr
    Dim s
    dim fso
    Dim currcert
    Dim fContinue
    Dim fFoundCert
    fFoundCert = False
    fContinue = True
    cnt = 0
    set fso = CreateObject("Scripting.FileSystemObject")
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "Creating Scripting.FileSystemObject", bVerbose
      Err.Clear             
    End If 
    Set txtstr = fso.OpenTextFile(filename)
    If Err.Number <> 0 Then
      LogError Err.Number, Err.Description, "OpenTextFile(" & filename & ")", bVerbose
      Err.Clear             
    End If 
    Do
        s = txtstr.ReadLine()
        'WScript.Echo s
        if txtstr.AtEndOfStream Then
            fContinue = false
        end if
        if strcomp(s, "-----BEGIN CERTIFICATE-----")=0 then
            cnt = cnt + 1
            if cnt = index Then
                fFoundCert = true
            Else
                fFoundCert = false
            End if
        elseif strcomp(s, "-----END CERTIFICATE-----")=0 then
            fFoundCert = false
        Elseif fFoundCert Then
            'must be reading certificate entry
            currcert = currcert & s & VbCrLf 
        end if

    Loop While fContinue
    GetCertFromFile= currcert
    
End Function

'******************************************************************************
'
' Function  : IsReDimed
'
' Synopsis  : Check to see if an array has any element.
'
' Parameter : Array - array to check.
'
' Return    : True if contains element, else False.
'
' Notes     : Should be false if the array was only Dim my_array() and nothing
'             else has been done to it. True if ReDimmed after that.
'
'******************************************************************************

Function IsReDimed (Array)

   On Error Resume Next
   
   Dim i : i = LBound(Array)
   If Err.Number = 0 Then
      IsReDimed = True
   Else
      IsReDimed = False
   End If
   
   On Error Goto 0
   
End Function ' End IsReDimed

'******************************************************************************
'
' Subroutine: LogError
'
' Synopsis  : Display, log or messagebox an error that just occurred.
'
' Parameter : ErrNum       - The error number.
'             ErrDesc      - The text description of the error.
'             LocationText - Text that describes what the program was attempting to do
'                            when the error occurred.
'             Verbose      - Display errors to the user as well.
'
' Notes :     Uses global WshShell object
'             The Err object also has the following properties but they do not typically 
'             have useful information in vbs:
'               Err.HelpContext Long integer pointing to a context in a help file.
'               Err.HelpFile    String containing the path to the relevant help file.
'               Err.Source      The name of the object causing the error.
'******************************************************************************
Sub LogError (ErrNum, ErrDesc, LocationText, Verbose)
  Dim ErrorString

  ' Create a string with all the information that we will want.
  '   application_name location_in_script (err#, hex_err#) description
  ErrorString = AppName & " " & LocationText & " (" & Cstr(ErrNum) & _
    ", 0x" & hex(ErrNum) & ") " & ErrDesc
  
  ' Sent it to the Application Event Log
  '   On Win 9x the event is added to the WSH.log in the Windows directory.
  WshShell.LogEvent LogEventERROR, ErrorString
  ' If is possible to get an error here, particularly on Win 9x if the %windir%\wsh.log file is locked.
  ' We could check for the error but what would we do with it? 
  ' We could display something on the screen or pop up a message box.

  If Verbose Then
    WScript.Echo "ERROR: " & ErrorString
  End If

End Sub ' End LogError

'******************************************************************************
Sub printUsage()
    WScript.Echo AppName
    WScript.Echo "Usage: cscript ctool.vbs [options]"
    WScript.Echo "Main commands"
    WScript.Echo "    /list              list the certificates in the store"
    WScript.Echo "      /subj <subjname> list the certificates that match <subjname>"
    WScript.Echo "      /sha1 <thumbprint> list the certificate that matches the thumbprint"
    WScript.Echo "      /app_policy <oid>list the certificates that match Application Policy <oid>"
    WScript.Echo "      /certpolicy <oid>list the certificates that match Certificate Policy <oid>"
    WScript.Echo "      /file <filename> list the certificate in the <filename>"
    WScript.Echo "    /show <thumbprint> displays a particular cert in a store"
    WScript.Echo "    /add <filename>    Adds the Base64 encoded certificates in <filename> to a store."
    WScript.Echo "    /add options:"
    WScript.Echo "      /noreplace 	       do not add the certificate if a certificate in the specified"
    WScript.Echo "                         store/storeloc already exists with the same thumbprint."
    WScript.Echo "    /del <thumbprint>  deletes the certificate with a matching SHA-1 thumbprint"
    WScript.Echo "      /del options:"
    WScript.Echo "      /validto <date>    deletes certificates that expire before <date>"
    WScript.Echo "      /delrevoked        deletes certificates if they are revoked"
    WScript.Echo "      /remove            Actually remove the certs"
    WScript.Echo ""
    WScript.Echo "    general options"
    WScript.Echo "    ------------"
    WScript.Echo "    /storeloc <store>  lm=Local Machine Store, cu = User Store. default = cu"
    WScript.Echo "    /verbose           can print additional information"
    WScript.Echo "    /store <store>     sets the certificate store.  Can be MY, CA, ROOT."
    WScript.Echo "                       default = MY"
    WScript.Echo ""
    WScript.Echo "Delete a certificate with a certain thumbprint in the user CA store:"
    WScript.Echo "   cscript ctool.vbs /del 7B02312BACC59EC388FEAE12FD277F6A9FB4FAC1 /remove"
    WScript.Echo "Display the certificates in a store:"
    WScript.Echo "   cscript ctool.vbs /list"
    WScript.Echo "   cscript ctool.vbs /list /store root"
    WScript.Echo "   cscript ctool.vbs /list /store my /storeloc lm"
    WScript.Echo "   cscript ctool.vbs /list /app_policy 1.3.6.1.4.1.311.10.3.4"
    WScript.Echo "Show the certificate UI for a certificate:"
    WScript.Echo "   cscript ctool.vbs /show 7E784A101C8265CC2DE1F16D47B440CAD90A1945 /store root"
    WScript.Echo "Import a cert into a store"
    WScript.Echo "   cscript ctool.vbs /add <filename>"
    WScript.Echo "      <filename> contains one or more Base64 encoded certificates"
    WScript.Echo ""
End Sub

